use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::AppStore::API::Shop qw ( CreateUnityScriptTagsApp );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );

#========================================================================================
# function         PatchUnityScriptTagsApp
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchUnityScriptTagsApp();
#----------------------------------------------------------------------------------------
# description      Creates the EpagesUIScriptTags app and adds the RegisterSession
#                   script-tag.
#========================================================================================
sub PatchUnityScriptTagsApp {
    my ($ShopAlias, $Force) = @_;

    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            # Unity shop
            if (!$Force && $Shop->get('UnityScriptTagsAppGUID')) {
                print 'Unity Storefront App for Shop ' . $ShopAlias . " got already renewed.\n"
                 . "Rerun with -force flag, to renew nevertheless. \n";
            } else {
                CreateUnityScriptTagsApp($Shop);
            }
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                # skip shops with already renews app
                next if (!$Force && $Shop->get('UnityScriptTagsAppGUID'));

                CreateUnityScriptTagsApp($Shop);
            };
            warn('Unity Storefront App renewal failed for shop ID ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Force;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'force'       => \$Force,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopid=s'    => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchUnityScriptTagsApp($ShopAlias, $Force); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Creates the EpagesUIScriptTags app and adds the RegisterSession
    script-tag.
Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -force      create a new app although a GUID is already defined
                for the shop
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );