#==============================================================================
# package      DE_EPAGES::Core::API::PerlTools
# description  This Module collects useful function, which uses only standard or cpan
#               modules. This Module initialize the randomizer via srand. Please
#               dont call srand again !!!
# state        public
#==============================================================================
package DE_EPAGES::Core::API::PerlTools;
use base qw ( Exporter );

use strict;
our @EXPORT_OK = qw(
    fcmp
    Compare
    Memoize

    unique
    string_unique
    trim
    isValid
    castToBoolean
    GenerateGUID
    GenerateShortGUID

    ElementOf
    Hex2Str
    IndexOf

    CopyStructure
    GetDoubleHashFromLoop
    GetLoopFromDoubleArray

    GetCallerStack
    GetCallerString
    GetCallerPackage
    GetCallerFunction

    GetArchName
    GetWindowsVersion

    GetLocalTimeZone
    GetLocalLocale
    GetCurrentDateTime
    AddDuration
    SubtractDuration
    GetIPAddress
    GetIPAddresses
    GetApplicationServerPort
    SetApplicationServerPort

    CreateNewPassWD
    CryptPassword

    CheckIpFormat
    Ip2Hex
    Str2Hex
    Str2PerlStrDoubleQuote
    Str2PerlStrSingleQuote
    GetMD5Digest

    CreateSecureCode
    VerifySecureCode
    CreateJsonWebToken

    UsePackage
    GetPackageFileName
    UnloadPackage
    IsA
);

use Carp;
use Config;
use Crypt::OpenSSL::RSA;
use Data::Dumper qw ( Dumper );
use DateTime::Format::Strptime;
use DateTime::TimeZone;
use Digest::MD5;
use Math::BigFloat;
use MIME::Base64;
use Mojo::JWT;
use Net::IPv6Addr;
use POSIX;
use Scalar::Util qw ( blessed );
use Sys::Hostname qw( hostname );
use URI::Escape qw( uri_escape_utf8 );

use DE_EPAGES::Core::API::Constants qw(CALLER_SUBNAME CALLER_FILENAME CALLER_LINE PERL516);
BEGIN {
## no critic qw(ProhibitStringyEval)
    if ($] < PERL516) {
        # use old packages for old perl
        eval 'use Socket  qw(AF_INET AF_INET6 SOCK_STREAM);'; die $@ if $@;
        eval 'use Socket6 qw(inet_pton getaddrinfo getnameinfo  NI_NUMERICHOST NI_NUMERICSERV);';  die $@ if $@;
    } else {
        # use new packages for new perl
        eval 'use Socket qw(AF_INET AF_INET6 SOCK_STREAM inet_pton getaddrinfo getnameinfo NI_NUMERICHOST NI_NUMERICSERV);';  die $@ if $@;
    }
## use critic
}

# initialize randomizer
srand( time() ^ ($$ + ($$ << 15)) ); ## no critic qw(ProhibitMagicNumbers)
my $APPLICATIONSERVER_PORT = 0;

BEGIN {
    $ENV{PERL_JSON_BACKEND} = 'JSON::PP'; ##no critic (RequireLocalizedPunctuationVars)
}

#========================================================================================
# function     UsePackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       UsePackage($PackageName);
#----------------------------------------------------------------------------------------
# description  Provides a save way to load a package instead of utilizing 'eval "use $Package"'
#               If $Package contains not a valid package name, this procedure dies with
#               a proper message. Otherwise, the package is loaded.
#----------------------------------------------------------------------------------------
# input        $PackageName | Name of the package to be loaded | string
#========================================================================================
sub UsePackage ($) {
    my ($Package) = @_;

    return if exists $INC{GetPackageFileName($Package)};

    if ($Package =~/^[a-zA-Z_]\w*(?:::\w+)*\z/x) {
        my $CallerPackage = caller(0);
        eval "package $CallerPackage; use $Package"; ##no critic (ProhibitStringyEval)
        if( $@ ) {
            my $Message = "$@";
            $Message =~ s/^(NativeDie: )+//;
            die "Can't compile package '$Package': $Message ; CallerPackage:'$CallerPackage' ";
        }
    }
    else {
        croak "Package '$Package' is not a valid package name!";
    }
}

#========================================================================================
# function     GetPackageFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PackageFileName = GetPackageFileName($PackageName);
#----------------------------------------------------------------------------------------
# description  return the name of corresponding file for a package
#----------------------------------------------------------------------------------------
# input        $PackageName | Name of the package to be loaded | string
# return       $PackageFileName | file name of package | string
#========================================================================================
sub GetPackageFileName ($) {
    my ($Package) = @_;

    return join( '/', split /::/, $Package ) . '.pm';
}

#========================================================================================
# function     IsA
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsA( $Object, $ClassName );
#----------------------------------------------------------------------------------------
# description  Provides a save way to check if a $Object is blessed into package $ClassName
#               or inherits from it. Should be used instead of UNIVERSAL->isa().
#----------------------------------------------------------------------------------------
# input        $Object | The object to check | object
# input        $ClassName | the name of the class | string
# return       $Result | true if $Object is blessed into package $ClassName or inherits
#                         from package $ClassName or false if not. | boolean
#========================================================================================
sub IsA {
    my ($Object, $ClassName) = @_;

    return 0 unless blessed( $Object );
    return $Object->isa( $ClassName );
}

#========================================================================================
# function     UnloadPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       UnloadPackage($PackageName);
#----------------------------------------------------------------------------------------
# description  Unloads a package to force it to be reloaded if needed next time.
#               If a package name is invalid, this procedure dies with a proper message.
#----------------------------------------------------------------------------------------
# input        $PackageName | package name | string
#========================================================================================
sub UnloadPackage{
    my ($Package) = @_;

    if ($Package =~/^[a-zA-Z_]\w*(?:::\w+)*\z/x){
        $Package =~ s/::/\//g;
        $Package .= '.pm';
        delete $INC{$Package} if exists $INC{$Package};
    } else {
        die "Package '$Package' is not a valid package name!";
    }
}

#========================================================================================
# function     GetCallerStack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aaStack = GetCallerStack($SkipCallers);
#----------------------------------------------------------------------------------------
# description  Stack information. See additional information at "perldoc -f caller".
#----------------------------------------------------------------------------------------
# input        $SkipCallers | ingore first elements of stack (default 0)(optional) | integer
# return       $aaStack | perl stack
#               <ul>
#                   <li>0 - means Package  -package name - string
#                   <li>1 - means FileName - file name - string
#                   <li>2 - means Line     - line number - integer
#                   <li>3 - means SubName  - function name - string
#                   <li>4 - means HasArgs  - has arguments (@_ is overwritten) - string
#                   <li>5 - means WantArray - caller wants an array - string
#                   <li>6 - means EvalText  - text of the "eval EXPR" statement - string
#                   <li>7 - means IsRequire -  created by a "require" or "use" statement - boolean
#               </ul> | ref.array.array
#========================================================================================
sub GetCallerStack (;$) {
    my ( $SkipCallers ) = @_;
    $SkipCallers = 0 unless $SkipCallers;

    my @CallStack;
    my $i = $SkipCallers + 1; # 1 for this function!
    while (my @Caller = caller($i++) ) {
        push @CallStack, \@Caller;
    }
    return \@CallStack;
}

#========================================================================================
# function     GetCallerString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CallerString = GetCallerString();
# syntax       $CallerString = GetCallerString( $aaCallers);
# syntax       $CallerString = GetCallerString( $aaCallers, $SkipCallers );
# syntax       $CallerString = GetCallerString( $aaCallers, $SkipCallers, $MaxCallers );
# example      $CallerString = GetCallerString( GetCallerStack() );
#----------------------------------------------------------------------------------------
# description  Returns a string representation of the caller stack.
#----------------------------------------------------------------------------------------
# input        $aaCallers | caller double array, see <function GetCallerStack> | ref.array.hash
# input        $SkipCallers | (optional, default=0) ignore first elements of stack | integer
# input        $MaxCallers | (optional) maximum call stack depth | integer
# return       $CallerString | nicely formatted caller stack | string
#========================================================================================
sub GetCallerString (;$$$) {
    my( $aaCallers, $SkipCallers, $MaxCallers ) = @_;
    $aaCallers = GetCallerStack() unless defined $aaCallers;
    $SkipCallers = 0 unless defined $SkipCallers;
    $MaxCallers = scalar @$aaCallers unless defined $MaxCallers;

    my $Message = '';
    foreach my $aCaller ( @{$aaCallers}[ $SkipCallers .. $MaxCallers - 1 ] ) {
        $Message .= sprintf "sub %s called at %s line %d.\n",
            $aCaller->[CALLER_SUBNAME], $aCaller->[CALLER_FILENAME], $aCaller->[CALLER_LINE];
    }
    return $Message;
}

#========================================================================================
# function     GetCallerPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package = GetCallerPackage($SkipCallers);
#----------------------------------------------------------------------------------------
# description  Called by package.
#----------------------------------------------------------------------------------------
# input        $SkipCallers | ingore first elements of stack (default 0)(optional) | integer
# return       $PackageName | package name | string
#========================================================================================
sub GetCallerPackage (;$) {
    my ($SkipCallers) = @_;
    $SkipCallers = defined $SkipCallers ? int($SkipCallers) : 0;

    my ( $module ) = caller( $SkipCallers);
    return $module;
}

#========================================================================================
# function     GetCallerFunction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FunctionName = GetCallerFunction($SkipCallers);
#----------------------------------------------------------------------------------------
# description  Called by function. Get your own function name, get name of your
#               caller function.
#----------------------------------------------------------------------------------------
# input        $SkipCallers | ingore first elements of stack (default 0)(optional) | integer
# return       $FunctionName | function name | string
#========================================================================================
sub GetCallerFunction (;$) {
    my ($SkipCallers) = @_;
    $SkipCallers = defined $SkipCallers ? int($SkipCallers) + 1: 1;

    my ( undef, undef, undef, $subname ) = caller( $SkipCallers );
    my ( $callmodule ) = caller( $SkipCallers - 1);
    return substr($subname,length($callmodule)+2); # remove module and :: from subname
}

#========================================================================================
# function     GetDoubleHashFromLoop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hhDoubleHash = GetDoubleHashFromLoop($ahLoop, $IdentKey);
#----------------------------------------------------------------------------------------
# description  Converts a loop (array/hash) to a double hash (hash,hash). Neccesary is
#               an identify field for the new key in the upper hash. Double entries are
#               lost, eg: $hhDoubleHash->{$_->{$IdentKey}} = $_ foreach @$ahLoop.
#
#----------------------------------------------------------------------------------------
# input        $ahLoop | loop | ref.array.hash.*
# input        $IdentKey | identifier for key (existing key of hashes in $ahLoop | string
# return       $hhDoubleHash | 2 dimension hash | ref.hash.hash.*
#========================================================================================
sub GetDoubleHashFromLoop ($$) {
    my ($ahLoop, $IdentKey) = @_;

    my %hDoubleHash;
    $hDoubleHash{$_->{$IdentKey}} = $_ foreach @$ahLoop;
    return \%hDoubleHash;
}

#========================================================================================
# function     GetLoopFromDoubleArray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLoop = GetLoopFromDoubleArray ($aaArray, $aKeys);
#----------------------------------------------------------------------------------------
# description  Converts a 2 dimension array into a loop. The second array will be replaced
#               by hash. Keys of hash are defined at second parameter.
#----------------------------------------------------------------------------------------
# input        $aaArray | 2D array | ref.array.array.*
# input        $aKeys | keys for inner hash | array.string
# return       $ahLoop | 2 dimension hash | ref.array.hash.*
#========================================================================================
sub GetLoopFromDoubleArray ($$) {
    my ($raArray, $raKeys) = @_;

    my @Result;
    foreach my $Row (@$raArray) {
        my %Row;
        @Row{@$raKeys} = @$Row;
        push @Result, \%Row;
    }
    return \@Result;
}

#========================================================================================
# function     ElementOf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsIncluded = ElementOf($Search, @List);
# syntax       $IsIncluded = ElementOf($SearchObject, @ObjectList);
#----------------------------------------------------------------------------------------
# description  Looks if an element is in the list (compare operator is 'eq' = string compare).
#               Looks if an object is in the list (compares the ids of the object).
#               Performance note: the expression ($Search ~~ @List) is much faster than
#               this function.
#----------------------------------------------------------------------------------------
# input        $Search | search word or search object | string or object
# input        @List | list of words or objects | array.string or array.object
# return       $IsIncluded | search word or object was found in list | boolean
#========================================================================================
sub ElementOf ($@) { ## no critic qw(RequireArgUnpacking)
    my $search = shift;

    if (blessed($search) and $search->can('id') ) {
        foreach my $item (@_) {
            return 1 if $item->id == $search->id;
        }
        return 0;
    }
    else {
        return $search ~~ @_ ? 1 : 0;
    }
}

#========================================================================================
# function     IndexOf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Position = IndexOf($Search, @List);
# syntax       $IsIncluded = ElementOf($SearchObject, @ObjectList);
#----------------------------------------------------------------------------------------
# description  Looks if an element is in the list (compare operator is 'eq' = string compare,
#               if element is an object compares the ids of the object) and
#               returns the posistion (0..) and -1 if not found.
#----------------------------------------------------------------------------------------
# input        $Search | search word or search object | string or object
# input        @List | list of words or objects | array.string or array.object
# return       $Position | position of search word | integer
#========================================================================================
sub IndexOf ($@) { ## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)
    my $search = shift;

    my $idx = 0;
    if (blessed($search) and $search->can('id') ) {
        foreach my $item (@_) {
            return $idx if $item->id == $search->id;
            $idx++;
        }
    } else {
        foreach my $item (@_) {
            return $idx if $item eq $search;
            $idx++;
        }
    }
    return -1;
}

#========================================================================================
# function     CopyStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Copy = CopyStructure($Structure);
#----------------------------------------------------------------------------------------
# description  Copies the full structure with objects, arrays, hashes, scalars but no
#               code references using Data::Dumper.
#----------------------------------------------------------------------------------------
# input        $Structure | structure to copy| ref.[^coderef]*
# return       $Copy | copy | ref.[^coderef]*
#========================================================================================
sub CopyStructure ($) {
    my ($rSource) = @_;
    local $Data::Dumper::Purity = 1;
    local $Data::Dumper::Useperl = 1;
    my $Dump = Dumper($rSource);
    my $VAR1; # used in Dump
    eval $Dump; ##no critic (ProhibitStringyEval)
    return $VAR1;
}

#========================================================================================
# function     CheckIpFormat
#----------------------------------------------------------------------------------------
# syntax       $ErrorMsg = CheckIpFormat($IpAddress);
# syntax       $ErrorMsg = CheckIpFormat($IpAddress, $AF);
#----------------------------------------------------------------------------------------
# example      $ErrorMsg = CheckIpFormat('123.120.34.254');
#----------------------------------------------------------------------------------------
# description  Tests if the parameter $IpAddress is an IP address of address family $AF.
#----------------------------------------------------------------------------------------
# input        $IpAddress | parameter for test | string
# input        $AF | (optional) address family (4 or 6) | number
# return       $ErrorMsg | error string | string
#========================================================================================
sub CheckIpFormat {
    my ($IpAddress, $AF) = @_;
    return 'VOID_IP_ADDRESS' unless $IpAddress =~ /[.:]/;
    return 'VOID_IP_ADDRESS' unless $IpAddress =~ /^[\da-f.:]{2,}$/ix;

    $AF = $IpAddress =~ /:/ ? 6 : 4 unless $AF;
    if( $AF == 4 ) {
        $IpAddress =~ /^(\d{1,3}\.){3}\d{1,3}$/x or return 'VOID_IPv4_ADDRESS';
        my @ValidAdressparts = grep { $_ > 255 } split(/\./, $IpAddress); ## no critic qw(ProhibitMagicNumbers)
        return 'VOID_IPv4_ADDRESS' if @ValidAdressparts;
    } else {
        # see http://tools.ietf.org/html/rfc4291
        Net::IPv6Addr::is_ipv6($IpAddress) or return 'VOID_IPv6_ADDRESS';
    }
    return;
}

#========================================================================================
# function     Ip2Hex
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IPHex = Ip2Hex($IP, $Length);
# example      Ip2Hex("1.2.3.4") -> 01020304;
# example      Ip2Hex("1.2.3.4", 4) ->  0304;
#----------------------------------------------------------------------------------------
# description  Returns the IP address as hex number.
#----------------------------------------------------------------------------------------
# input        $IP | IPv4 or IPv6 IP address string, e.g. 1.2.3.4 | string
# input        $Length | cut the hex number to $Length characters
# return       $IPAddress | IPv4 or IPv6 address as hex number | string
#========================================================================================
sub Ip2Hex ($;$) {
    my ($IP, $Length) = @_;
    croak 'missing parameter IP' unless defined $IP;

    $Length = 0 unless $Length;
    my $Family = $IP =~ /:/ ? AF_INET6 : AF_INET;
    my $NumAddr = inet_pton($Family, $IP);
    croak "inet_pton($Family, $IP) failed" unless defined $NumAddr;

    return uc( substr( unpack('H*', $NumAddr), -$Length ) );
}

#========================================================================================
# function     GetIPAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IPAddress = GetIPAddress();
# syntax       $IPAddress = GetIPAddress($Address, $AF);
#----------------------------------------------------------------------------------------
# description  Returns the first IP address from $Address if multiple addresses are used.
#               If address family $AF is omitted, 6 (IPv6) is used.
#               If $Address is omitted, hostname() is used.
#               The information is memorized until the program ends.
#----------------------------------------------------------------------------------------
# input        $Address | (optional) hostname or ip address | string
# input        $AF | (optional) address family: 4 (IPv4) or 6 (IPv6) | number
# return       $IPAddress | IPv4/IPv6 address | string
#========================================================================================
sub GetIPAddress(;$$) { return _GetIPAddress(@_); } ## no critic qw( RequireArgUnpacking )

*_GetIPAddress = Memoize( sub {
    my ($Address, $AF) = @_;
    if ($Address && !CheckIpFormat($Address)) {
       my $IpVersion = $Address =~ /:/ ? 6 : 4;
       return $Address if not $AF or $AF == $IpVersion;
    }
    my $IpHash = {};
    $IpHash = GetIPAddresses($Address) or return;

    my $EnableIPv6 = $ENV{EPAGES_ENABLE_IPv6};
    $EnableIPv6 =~ s/\s//g if $EnableIPv6;

    if (!$AF && $EnableIPv6 && defined $IpHash->{6}) {
        $AF = scalar @{$IpHash->{6}} ? 6 : 4;
    } elsif (!$AF) {
        $AF = 4;
    }
    return unless defined $IpHash->{$AF};
    return unless scalar @{$IpHash->{$AF}};
    return $IpHash->{$AF}[0];
});

#========================================================================================
# function     Memoize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FastSub = Memoize($Sub);
# example      *Fast_Fib = Memoize( \&Fib );
#               print Fast_Fib(30);
#----------------------------------------------------------------------------------------
# description  Creates a faster version of a sub that caches all results.
#               Caveats:
#                   * the return value must be a single scalar
#                   * the function parameters cannot be references
#                   * the cache key is join('-',@_), i.e. there may be clashes if parameters
#                       contain a '-' character
#----------------------------------------------------------------------------------------
# input        $Sub | slow function | ref.code
# return       $FastSub | fast function | ref.code
#========================================================================================
sub Memoize {
    my ($Sub) = @_;

    my %CACHE; # closure
    return sub {
        foreach( @_ ) {
            die "Memoize: parameter $_ is not scalar" if ref $_;
        }

        my $CacheString = join('-', map { $_ // '(undef)' } @_);
        if( exists $CACHE{$CacheString} ) {
            return $CACHE{$CacheString};
        }
        else {
            return $CACHE{$CacheString} = $Sub->(@_);
        }
    };
}

#========================================================================================
# function     GetIPAddresses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IpHash = GetIPAddresses($Address);
#----------------------------------------------------------------------------------------
# description  Returns a hash of all IP addresses for $Address (IP or hostname).
#               If $Address is omitted, hostname() is used.
#               The information is memorized until the program ends.
#----------------------------------------------------------------------------------------
# input        $Address |IPv4/IPv6 address or hostname | string
# return       $IpHash | hash of IP address arrays with AF keys (4 or 6)
#========================================================================================
sub GetIPAddresses {
    my ($Address) = @_;

    my @Addresses;
    if (not defined $Address) {
        my $HostName = hostname();
        push @Addresses, $HostName;
        push @Addresses, $1 if $HostName =~ /(.+?)\./;
    } else {
        @Addresses = ($Address);
    }

    my $IpHash = {};

    foreach my $Address (@Addresses) {
        foreach my $AF (AF_INET, AF_INET6) {
            if ($] < PERL516) {
                # use old getaddrinfo/getnameinfo from Socket6
                my @Result = getaddrinfo($Address, '', $AF, SOCK_STREAM);
                my $SockAddr;
                while (scalar(@Result) >= 5) {
                    (undef, undef, undef, $SockAddr, undef, @Result) = @Result;
                    my $IP = (getnameinfo($SockAddr, NI_NUMERICHOST|NI_NUMERICSERV))[0];
                    push @{$IpHash->{$AF == AF_INET6 ? 6 : 4}}, $IP
                        if $IP =~ /^[0-9a-fA-F:.]+$/;
                }
            } else {
                # use new getaddrinfo/getnameinfo from Socket
                my $hints = {socktype => SOCK_STREAM, family => $AF};
                my (undef, @Result) = getaddrinfo($Address, '', $hints);
                foreach my $Item (@Result) {
                    my $IP = (getnameinfo($Item->{addr}, NI_NUMERICHOST|NI_NUMERICSERV))[1];
                    push @{$IpHash->{$AF == AF_INET6 ? 6 : 4}}, $IP
                        if $IP =~ /^[0-9a-fA-F:.]+$/;
                }
            }
        }
    }
    return $IpHash;
}

#========================================================================================
# function     GetApplicationServerPort
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ApplicationServerPort = GetApplicationServerPort();
#----------------------------------------------------------------------------------------
# description  Returns the port of the application server.
#----------------------------------------------------------------------------------------
# return       $ApplicationServerPort | as port 100043 | integer
#========================================================================================
sub GetApplicationServerPort() {
    return $APPLICATIONSERVER_PORT;
}

#========================================================================================
# function     SetApplicationServerPort
# state        private
#----------------------------------------------------------------------------------------
# syntax       SetApplicationServerPort($ApplicationServerPort);
#----------------------------------------------------------------------------------------
# description  Sets the gloabl variable for <function GetApplicationServerPort>.
#----------------------------------------------------------------------------------------
# input       $ApplicationServerPort | as port 100043 | integer
#========================================================================================
sub SetApplicationServerPort($) {
    my ($Port) = @_;
    $APPLICATIONSERVER_PORT = $Port;
}

#========================================================================================
# function     GenerateGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GUID = GenerateGUID();
#----------------------------------------------------------------------------------------
# description  Returns 36-character long strings containing GUID (UUID). These look something like:
#               b09e53f4-60e9-41a6-8b30-526b69c28b5d.
#----------------------------------------------------------------------------------------
# return       $GUID | general unique identifier | string
#========================================================================================
sub GenerateGUID () {

    ## no critic qw(ProhibitMagicNumbers)
    my ($time_low, $time_mid, $time_hi_and_version, $clock_seq_hi_and_reserved, $clock_seq_low)
        = (time, rand(256*256), rand(256*256), rand(256), rand(256));
    return sprintf ( '%8.8X-%4.4X-%4.4X-%2.2X%2.2X-%s%2.2X%2.2X', $time_low, $time_mid,
                     $time_hi_and_version, $clock_seq_hi_and_reserved, $clock_seq_low,
                     Ip2Hex(GetIPAddress(),8), $$ % 256, rand(256) % 256 );
    ## use critic
}

#========================================================================================
# function     GenerateShortGUID
#----------------------------------------------------------------------------------------
# syntax       $ShortGUID = GenerateShortGUID();
#----------------------------------------------------------------------------------------
# description  returns a 20 char GUID which is shorter than the original 36 chars
#----------------------------------------------------------------------------------------
# return       $GUID | general unique identifier | string
#========================================================================================
sub GenerateShortGUID {
    return sprintf('%08X%s%02X%02X', time(), Ip2Hex(GetIPAddress(),8), $$ % 256, rand(256) % 256 ); ## no critic qw(ProhibitMagicNumbers)
}

#========================================================================================
# function     isValid
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Valid = isValid($String);
#----------------------------------------------------------------------------------------
# description  Is string defined and not empty ?
#----------------------------------------------------------------------------------------
# input        $String | test string | string
# return       $Valid | parameter is simple valid | boolean
#========================================================================================
sub isValid ($) { ## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)
    return (defined($_[0]) && ($_[0] ne ''));
}
#========================================================================================
# function     castToBoolean
# state        public
#----------------------------------------------------------------------------------------
# syntax       castToBoolean( $Value );
#----------------------------------------------------------------------------------------
# description  cast any $Value into an predictable boolean output
#               mapping table:
#               * undef               = false
#               * ''                  = false # As a boolean this is false, as a String it would be true
#               * '0'                 = false # As a boolean this is false, as a String it would be true
#               * 'undef'             = true  # This is a normal String
#               * 1                   = true
#               * 0                   = false
#               * []                  = true # The array pointer is defined, although the list is empty
#               * @a;                 = false
#               * @a = ();            = false
#               * @a = (undef);       = true # This is a list with one element, which is undef
#               * @a = ('one');       = true
#               * {}                  = true # The hash pointer is defined, although the Hash is empty
#               * %a;                 = false
#               * %a = ();            = false
#               * %a = ('' => undef); = true # This is a hash with one key-value pair, which is undef
#               * %a = ('a' => 'b');  = true
#----------------------------------------------------------------------------------------
# input        $Value | any type
# return       $ValueAsBoolean | 1 or undef (not the keyword 'undef') | boolean
#========================================================================================
sub castToBoolean($){## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)
    # All different Types will be handled in scalar context, because of the expected parameter type ($)
    # If it is called with an @Array, $_[0] will contain the Size of the Array
    # If it is called with an %Hash,  $_[0] will contain the used buckets (it is about allocated memory)
    #   which is a defined string and therefore -> true
    if($_[0]){
        return 1;
    } else {
        if(defined($_[0])){
            if($_[0] eq ''){
                return;
            }
            if($_[0] == 0){
                return;
            }
            return 1;
        }
        return;
    }
}

#========================================================================================
# function     unique
# state        public
#----------------------------------------------------------------------------------------
# syntax       @UniqueValues = unique @Values;
# example      print unique( 3,2,1,2,3 )
#----------------------------------------------------------------------------------------
# description  Removes duplicates from a list and preserves the sort order from the
#               beginning.
#----------------------------------------------------------------------------------------
# input        @Values | list of values with duplicates | array
# return       @UniqueValues | list of unique values | array
#========================================================================================
sub unique(@) { ## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)

    my %UniqueValues;
    my @UniqueValues;
    local $_ = undef;
    foreach my $val ( @_ ) {
        push @UniqueValues, $val unless exists $UniqueValues{$val};
        $UniqueValues{$val} = undef;
    }
    return @UniqueValues;
}

#========================================================================================
# function     string_unique
# state        public
#----------------------------------------------------------------------------------------
# syntax       @UniqueValues = string_unique @Values;
# example      print unique( "3", $a )
#----------------------------------------------------------------------------------------
# description  Removes duplicates from a list and preserves the sort order from the
#               beginning. The string representation of each value "$Value" is used to
#               find duplicates. Useful for strings but also for DateTime objects.
#----------------------------------------------------------------------------------------
# input        @Values | list of values with duplicates | array.*
# return       @UniqueValues | list of unique values | array
#========================================================================================
sub string_unique(@) { ## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)

    my %UniqueValues;
    my @UniqueValues;
    local $_ = undef;
    foreach my $val ( @_ ) {
        push @UniqueValues, $val unless exists $UniqueValues{"$val"};
        $UniqueValues{"$val"} = undef;
    }
    return @UniqueValues;
}


#========================================================================================
# function     trim
# state        public
#----------------------------------------------------------------------------------------
# syntax       @TrimmedValues = trim( @Values );
# example      print trim "  A", " B", "\t   C\n";
#----------------------------------------------------------------------------------------
# description  Removes leading and training whitespaces from a list of values.
#----------------------------------------------------------------------------------------
# input        @Values | list of values with leading or trailing whitespace | array
# return       @TrimmedValues | list of trimmed values | array
#========================================================================================
sub trim(@) { ## no critic qw(RequireCommentForExportedFunctions RequireArgUnpacking)

    return map {
        local $_ = $_;
        s/^\s+//;
        s/\s+$//;
        $_;
    } @_;
}

#========================================================================================
# function     GetArchName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ArchName = GetArchName();
#----------------------------------------------------------------------------------------
# description  Returns architecture name and 32 bit version of architecture name
#----------------------------------------------------------------------------------------
# return       ($ArchName, $ArchName32) | architecture name, one of:
#                           sun4-solaris
#                           sun4-solaris64
#                           i86pc-solaris
#                           i86pc-solaris64
#                           i586-linux
#                           x86_64-linux
#                           i586-MSWin32
#                           x86_64-MSWin32             | string
#========================================================================================
sub GetArchName () {
    my ($ArchName, $ArchName32);
    if ( $Config{osname} eq 'solaris' ) {
        if ( $Config{byteorder} eq '12345678' ) {
            $ArchName = 'i86pc-solaris64';
            $ArchName32 = 'i86pc-solaris';
        } elsif  ( $Config{byteorder} eq '87654321' ) {
            $ArchName = 'sun4-solaris64';
            $ArchName32 = 'sun4-solaris';
        } elsif ( $Config{byteorder} eq '1234' ) {
            $ArchName =  'i86pc-solaris';
            $ArchName32 = $ArchName;
        } elsif ( $Config{byteorder} eq '4321' ) {
            $ArchName =  'sun4-solaris';
            $ArchName32 = $ArchName;
        }
    } elsif ( $Config{byteorder} eq '12345678' )  {
        $ArchName = "x86_64-$Config{osname}";
        $ArchName32 = "i586-$Config{osname}";
    } elsif ( $Config{byteorder} eq '1234' )  {
        $ArchName = "i586-$Config{osname}";
        $ArchName32 = $ArchName;
    }
    return ($ArchName, $ArchName32);
}

#========================================================================================
# function     GetWindowsVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($VersionName, $VersionNumber) = GetWindowsVersion();
#----------------------------------------------------------------------------------------
# description  Returns Windows version name and number
#----------------------------------------------------------------------------------------
# return       ($VersionName, $VersionNumber) | number as 6.0.6001; name, list of:
#                           Windows NT 3
#                           Windows NT 4
#                           Windows 95
#                           Windows 98
#                           Windows ME
#                           Windows NT 5
#                           Windows 2000
#                           Windows XP
#                           Windows .NET
#                           Windows 2003
#                           Windows Vista
#                           Windows 2008
#                           Windows 7          | string
#========================================================================================
sub GetWindowsVersion () {
    my ($VersionName, $VersionNumber);
    return ($VersionName, $VersionNumber) unless $Config{osname} eq 'MSWin32';

    $VersionNumber = `ver`;
    chomp($VersionNumber);
    $VersionNumber =~ s/.*\[.*?([0-9][0-9.]*).*/$1/sx;

    my %WindowsVersion = (
        '3.'        => [ 'Windows NT 3' ],
        '4.00.1381' => [ 'Windows NT 4' ],
        '4.0'       => [ 'Windows 95' ],
        '4.10'      => [ 'Windows 98' ],
        '4.90'      => [ 'Windows ME' ],
        '5.00.1'    => [ 'Windows NT 5' ],
        '5.00.2'    => [ 'Windows 2000' ],
        '5.1.'      => [ 'Windows XP' ],
        '5.2.35'    => [ 'Windows .NET' ],
        '5.2.36'    => [ 'Windows .NET' ],
        '5.2.3718'  => [ 'Windows .NET' ],
        '5.2.37'    => [ 'Windows 2003' ],
        '6.0.5'     => [ 'Windows Vista' ],
        '6.0.6000'  => [ 'Windows Vista' ],
        '6.0.6001'  => [ 'Windows 2008' ],
        '6.0.6002'  => [ 'Windows 2008', 'Windows Vista' ],
        '6.1.7600'  => [ 'Windows 7', 'Windows 2008' ],
        );

    if (defined $WindowsVersion{$VersionNumber}) {
        $VersionName = $WindowsVersion{$VersionNumber};
    } else {
        foreach my $Key (sort keys %WindowsVersion) {
            if ($VersionNumber =~ /^\Q$Key\E/) {
                $VersionName = $WindowsVersion{$Key};
                last;
            }
        }
    }

    return ($VersionName, $VersionNumber);
}

#========================================================================================
# function     GetLocalTimeZone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TimeZone = GetLocalTimeZone();
#----------------------------------------------------------------------------------------
# description  Returns the name of the server time zone. Uses the environment variables
#               EPAGES_TZ or TZ.
#----------------------------------------------------------------------------------------
# return       $TimeZone | local time zone name (example: "Europe/Berlin") | string
#========================================================================================
sub GetLocalTimeZone() { return _GetLocalTimeZone(); }

*_GetLocalTimeZone = Memoize( sub {
    local $ENV{TZ} = $ENV{TZ} // $ENV{'EPAGES_TZ'};

    my $tz = eval { DateTime::TimeZone->new( name => 'local' ); };
    return $tz->name unless $@;

    carp 'no valid time zone information found, please set environment variable EPAGES_TZ.';

    my $timer = time;
    my ($sec,$min,$hour,$mday,$mon,$year) = localtime($timer);
    my $loc_time = sprintf '%4d-%02d-%02d %02d:%02d:%02d', $year + 1900,$mon + 1,$mday,$hour,$min,$sec; ## no critic qw(ProhibitMagicNumbers)

    ($sec,$min,$hour,$mday,$mon,$year) = gmtime($timer);
    my $gmt_time = sprintf '%4d-%02d-%02d %02d:%02d:%02d', $year + 1900,$mon + 1,$mday,$hour,$min,$sec; ## no critic qw(ProhibitMagicNumbers)

    my $Strp = DateTime::Format::Strptime->new(
        pattern     => '%F %T',
        locale      => 'en_AU',
        time_zone   => '+0000',
    );
    my $d = $Strp->parse_datetime($loc_time) - $Strp->parse_datetime($gmt_time);
    return sprintf('%s%02d:%02d', $d->is_negative ? '-' : '+' , $d->hours,$d->minutes);
});

#========================================================================================
# function     GetLocalLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $locale = GetLocalLocale();
#----------------------------------------------------------------------------------------
# description  Returns the local locale (germany 'de') DateTime::Locale of the
#               application server machine.
#               Under windows operating system returns the full name of languange
#               and country (e.q. "German Germany") wh
#----------------------------------------------------------------------------------------
# return       $locale | locale id (de_AT, en_GB) | string
#========================================================================================
sub GetLocalLocale  () { return _GetLocalLocale(); }

*_GetLocalLocale = Memoize( sub {
    my $PosixLocale = setlocale(LC_CTYPE);
    $PosixLocale =~ s/\..*$//;
    $PosixLocale =~ s/_/ /g unless $PosixLocale =~ /^.{2}_.{2}$/;
    return $PosixLocale;
});

#========================================================================================
# function     GetCurrentDateTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DateTime = GetCurrentDateTime();
#----------------------------------------------------------------------------------------
# description  Returns the current time with server time_zone.
#----------------------------------------------------------------------------------------
# return       $DateTime | current time | DateTime object
#========================================================================================
sub GetCurrentDateTime () {
    return DateTime->now('time_zone'=>GetLocalTimeZone());
}

#========================================================================================
# function     AddDuration
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DateTime = AddDuration($OldDateTime, $DurationUnit, $Duration);
# $example      $DateTime = AddDuration($OldDateTime, 'days', 1);
#----------------------------------------------------------------------------------------
# description  Adds a duration to a datetime object and returns the new one.
#----------------------------------------------------------------------------------------
# input        $DateTime | date time | DateTime object
# input        $DurationUnit | daration unit (minutes, hours, days, months, ...) | string
# input        $Duration | daration value | integer
# return       $DateTime | date time after duration | DateTime object
#========================================================================================
sub AddDuration ($$$) {
    my ($OldDateTime, $DurationUnit, $Duration) = @_;

    my $DateTime = $OldDateTime->clone;
    my $TimeZone = $DateTime->time_zone;
    $DateTime->set_time_zone('UTC');
    $DateTime->add($DurationUnit=>$Duration);
    return $DateTime->set_time_zone($TimeZone);
}

#========================================================================================
# function     SubtractDuration
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DateTime = SubtractDuration($DateTime, $DurationUnit, $Duration);
# $example      $DateTime = SubtractDuration($DateTime, 'days', 1);
#----------------------------------------------------------------------------------------
# description  Subtracts a duration of a datetime object and returns the new one.
#               NOTE: Unlike <function AddDuration> this function also modifies the input
#               paramter $DateTime.
#----------------------------------------------------------------------------------------
# input        $DateTime | date time | DateTime object
# input        $DurationUnit | daration unit (minutes, hours, days, months, ...) | string
# input        $Duration | daration value | integer
# return       $DateTime | date time after duration | DateTime object
#========================================================================================
sub SubtractDuration ($$$) {
    my ($DateTime, $DurationUnit, $Duration) = @_;

    my $TimeZone = $DateTime->time_zone;
    $DateTime->set_time_zone('UTC');
    $DateTime->subtract($DurationUnit=>$Duration);
    return $DateTime->set_time_zone($TimeZone);
}

#========================================================================================
# function     fcmp
# state        public
#----------------------------------------------------------------------------------------
# syntax       $result = fcmp($Float1, $Float2, $Precision);
# example      my $a = 0.00011;
#               my $b = 1.10 - 1.09989;
#               printf "%d != %d\n", fcmp($a,$b), $a <=> $b;
#----------------------------------------------------------------------------------------
# description  Compares two float numbers (avoid numeric problems, see example).
#----------------------------------------------------------------------------------------
# input        $Float1 | first float number | float
# input        $Float2 | second float number | float
# input        $Precision | round to number of valid digits (optional 10) | int
# return       $result | 0 if pass / -1 if smaller / 1 if greater / undef if one is no float number | int
#========================================================================================
sub fcmp ($$;$) {
    my ($NStr1, $NStr2, $Precision) = @_;
    $Precision = 10 unless defined $Precision; ## no critic qw(ProhibitMagicNumbers)

    my $bigFloat1 = Math::BigFloat->new($NStr1)->ffround(-$Precision); # round to 8 digits
    my $bigFloat2 = Math::BigFloat->new($NStr2)->ffround(-$Precision); # round to 8 digits

    return undef if $bigFloat1 eq 'NaN' || $bigFloat2 eq 'NaN';
    return $bigFloat1->fcmp($bigFloat2);
}

#========================================================================================
# function     Compare
# state        public
#----------------------------------------------------------------------------------------
# syntax       $result = Compare($Type, $a, $b);
#----------------------------------------------------------------------------------------
# description  Different compares between a and b depending on the type of both elements.
#               Comparing two objects ignores input $Type, both objects must implement method id().
#               Function dies if comparison is not supported for given input. Note older
#               versions didn't check parameter $Type properly, instead raising an error,
#               undef was returned as result for the broken comparison.
#----------------------------------------------------------------------------------------
# input        $Type | String, LocalizedString, File, LocalizedFile, Money, Float, Boolean, Integer, DateTime or ClassName | string
# input        $a | value 1 | *
# input        $b | value 2 | *
# return       $result | 0 if $a equals $b / -1 if $a is smaller / 1 if $a is greater | int
#========================================================================================
sub Compare {
    my ($Type, $a, $b) = @_;

    return 0 unless defined $a or defined $b;
    return 1 if defined $a and not defined $b;
    return -1 if not defined $a and defined $b;
    return $a cmp $b            if $Type eq 'String' or $Type eq 'LocalizedString' or $Type eq 'File' or $Type eq 'LocalizedFile' or $Type eq 'EMail';
    return fcmp($a,$b)          if $Type eq 'Money' or $Type eq 'Float';
    return $a <=> $b            if $Type eq 'Boolean' or $Type eq 'Integer' or  blessed($a) and blessed($b) and blessed($a) eq 'DateTime' and blessed($b) eq 'DateTime';
    return $a->id <=> $b->id    if blessed($a) and blessed($b) and $a->can('id') and $b->can('id');

    my $Msg = "Input not suitable for comparison (Type=$Type, a=$a, b=$b)."
            . ' See http://helpcenter.epages.com/Doc/current/epages/API/DE_EPAGES/Core/API/PerlTools.pm.html#Compare.'
            . ' ePages issue number is EPG-27948.';
    die $Msg;
}

#========================================================================================
# function     CreateNewPassWD
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Password = CreateNewPassWD($UserName, $Length);
#----------------------------------------------------------------------------------------
# description  Obsolete. Use E_EPAGES::Core::API::Password::GenerateNewPassword.
#               Generates a plain random word containing only alphanumeric characters.
#----------------------------------------------------------------------------------------
# input        $UserName | unused | string
# input        $Length | (optional, default = 8) password length | integer
# return       $Password | generated password | string
#========================================================================================
sub CreateNewPassWD {
    my ($UserName, $Length) = @_;

    UsePackage('DE_EPAGES::Core::API::Password');
    return DE_EPAGES::Core::API::Password::GenerateNewPassword($Length);
}

#========================================================================================
# function     CryptPassword
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Hash = CryptPassword($Password);
#----------------------------------------------------------------------------------------
# description  Obsolete. Use DE_EPAGES::Core::API::Password::CryptPassword.
#               Generates a bcrypt hash of the password.
#----------------------------------------------------------------------------------------
# input        $Password | plain password | string
# input        $Salt | Obsolete, usage will cause a die. | string
# return       $Hash | bcrypt hashed password | string
#========================================================================================
sub CryptPassword {
    my ($Password, $Salt) = @_;

    die 'use DE_EPAGES::Core::API::Password::CheckPassword to verify a password' if defined $Salt;

    UsePackage('DE_EPAGES::Core::API::Password');
    return DE_EPAGES::Core::API::Password::CryptPassword($Password);
}

#========================================================================================
# function     Str2Hex
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Hex = Str2Hex($String);
#----------------------------------------------------------------------------------------
# description  Convers each byte in a string to a hexadecimal code.
# example      print Str2Hex('12ABab'); # prints '313241426162'
#----------------------------------------------------------------------------------------
# input        $String | any string | string
# return       $Hex | hex representation of the string | string
#========================================================================================
sub Str2Hex {
    my ($String) = @_;
    return undef unless defined $String;

    my $Hex = '';
    for (my $i=0; $i<length($String); $i++) {
        $Hex .= sprintf '%02X', ord(substr($String,$i,1));
    }
    return $Hex;
}

#========================================================================================
# function     Hex2Str
# state        public
#----------------------------------------------------------------------------------------
# syntax       $String = Hex2Str(HexString);
#----------------------------------------------------------------------------------------
# description  Converts a hexadecimal code to a string.
# example      print Hex2Str('313241426162'); # prints '12ABab'
#----------------------------------------------------------------------------------------
# input        $HexString | hex representation of the string | string
# return       $String | any string | string
#========================================================================================
sub Hex2Str {
    my ($HexString) = @_;
    return undef unless defined $HexString;

    my $String = '';
    for (my $i=0; $i<length($HexString); $i+=2) {
        $String.=chr(hex(substr($HexString,$i,2)));
    }
    return $String;
}

#========================================================================================
# function     Str2PerlStrDoubleQuote
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OutString = Str2PerlStrDoubleQuote($InString);
# example      eval "\$Var1 = \"" . Str2PerlStrDoubleQuote('some string also @@$$ allowed') ."\";";
#----------------------------------------------------------------------------------------
# description  Quotes the string that it can be used in a perl eval. The eval string includes
#               a variable assignment with double quotes (see example).
#----------------------------------------------------------------------------------------
# input        $InString | string the includes \\n"%$@ | string
# return       $OutString | quoted string | string
#========================================================================================
sub Str2PerlStrDoubleQuote {
    local ($_) = @_;
    s/\\/\\\\/g;
    s/\n/\\n/g;
    s/"/\\"/g;
    s/[\@\$\%]/\\$&/g;
    return $_;
}

#========================================================================================
# function     Str2PerlStrSingleQuote
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OutString = Str2PerlStrSingleQuote($InString);
# example      eval "\$Var1 = '" . Str2PerlStrSingleQuote('some string also @@$$ allowed') ."';";
#----------------------------------------------------------------------------------------
# description  Quotes the string that it can be used in a perl eval.The eval string includes
#               a variable assignment with single quotes (see example).
#----------------------------------------------------------------------------------------
# input        $InString | string the includes \\n"%$@ | string
# return       $OutString | quoted string | string
#========================================================================================
sub Str2PerlStrSingleQuote {
    local ($_) = @_;
    s/\\/\\\\/g;
    s/\n/\\n/g;
    s/'/\\'/g;
    return $_;
}

#========================================================================================
# function     LicensePublicKey
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PublicKey = LicensePublicKey();
#----------------------------------------------------------------------------------------
# description  Returns the public key to verify the license signature
#----------------------------------------------------------------------------------------
# return       $PublicKey | public rsa key | string
#========================================================================================
sub LicensePublicKey {
    return <<END_SIG
-----BEGIN RSA PUBLIC KEY-----
MIIBCgKCAQEAwnxTOd4faWoWtud/LOw8CvdxHAkt4wpdFw9RUdw2pbuJNrWsMw16
HFdD1M4yj3uNrxzyMF5gab37PnMNKeusZdduKUCvMY7kK33N3kk54lpKMV74Mgkr
4ymlF7cZW84t38OvVQkDGXtZiK9YhTGkXg3W8i1G6C7yeCVVbQPbGKq6xNkIfduz
U/tYc9UZOauQTTz4dLF1sR6A/0cSws9eNReIfhl2NqMbGvWJrIThYIwm282wdWfs
wTzsDu//FwlCLiD8Lv0I35+W253FHkCsxeUtrYILBhK6ZoHQkMIP0txQLrY0SA53
gL7W2VRY2XO/GfXzmn3CDPyV5VARppzG2wIDAP//
-----END RSA PUBLIC KEY-----
END_SIG
}

#========================================================================================
# function     VerifyLicenseContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Verified = VerifyLicenseContent($Content, $Signature);
#----------------------------------------------------------------------------------------
# description  Returns if the license is content was signed by the counterpart
#               (private key) of the public key.
#----------------------------------------------------------------------------------------
# input        $Content | content to get verified | string
# input        $Signature | signature of content (base64 encoded) | string
# return       $Verified  | content is correct signed | boolean
#========================================================================================
sub VerifyLicenseContent {
    my ($Content, $Signature) = @_;
    return Crypt::OpenSSL::RSA->new_public_key(LicensePublicKey())->verify($Content, decode_base64($Signature));
}

#========================================================================================
# function     GetMD5Digest
#----------------------------------------------------------------------------------------
# syntax       $HexDigest = GetMD5Digest($rContent);
#----------------------------------------------------------------------------------------
# example      my $HexDigest = GetMD5Digest(GetFileContent("/var/log/error.log"));
#----------------------------------------------------------------------------------------
# description  Returns the md5 hex digest of content
#----------------------------------------------------------------------------------------
# input        $rContent | content of file | ref.string
# return       $HexDigest | md5 hex digest ( see perldoc Digest::MD5) | string
#========================================================================================
sub GetMD5Digest ($) {
    my ($rContent) = @_;
    return Digest::MD5->new->add($$rContent)->hexdigest;
}

#========================================================================================
# function     CreateSecureCode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Code = CreateSecureCode($Content);
#----------------------------------------------------------------------------------------
# example      my $Code = CreateSecureCode("/var/log/error.log");
#----------------------------------------------------------------------------------------
# description  Returns an secure code of content
#----------------------------------------------------------------------------------------
# input        $Content | text to make sure that it wouldnt be changed | string
# return       $Code | secure code for content (to use <function VerifySecureCode>) | string
#========================================================================================
sub CreateSecureCode ($) {
    my ($Content) = @_;

    my $Encoded = encode_base64(uri_escape_utf8($Content));
    return GetMD5Digest(\$Encoded);
}

#========================================================================================
# function     VerifySecureCode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsValid = VerifySecureCode($Content);
#----------------------------------------------------------------------------------------
# example      my $IsValid = VerifySecureCode("/var/log/error.log", CreateSecureCode("/var/log/error.log"));
#----------------------------------------------------------------------------------------
# description  Returns if the given secure code matches to the content.
#----------------------------------------------------------------------------------------
# input        $Content | text to make sure that it wouldnt be changed | string
# input        $Code | secure code for content (to use <function VerifySecureCode>) | string
# return       $IsValid | code is valid for content | string
#========================================================================================
sub VerifySecureCode ($$) {
    my ($Content, $Code) = @_;

    return CreateSecureCode($Content) eq $Code ? 1 : 0;
}

#========================================================================================
# function     CreateJsonWebToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateJsonWebToken($Lifetime, $Subject, $Scope, $Issuer, $Auditor, $Secret)
#----------------------------------------------------------------------------------------
# description  Returns a hex-encoded JSON web token, with HMACSHA256 as algorithm.
#----------------------------------------------------------------------------------------
# input        $Lifetime | the token's lifetime in seconds| integer
# input        $Subject | the token's subject (e.g. ShopAlias)| string
# input        $Scope | the token's scope (e.g. editor or provisioning)| string
# input        $Issuer | the token's issuer (e.g. http://epages.com/epages6)| string
# input        $Auditor | the token's auditor (e.g. http://epages.com/unity)| string
# input        $Secret | shared secret used for the token's encryption | string
#----------------------------------------------------------------------------------------
# return       $WebToken | encoded JSON web token | string
#========================================================================================
sub CreateJsonWebToken {
    my ($Lifetime, $Subject, $Scope, $Issuer, $Auditor, $Secret) = @_;

    my $WebToken = Mojo::JWT->new({
        'algorithm' => 'HS256',
        'expires'   => AddDuration(GetCurrentDateTime(), 'seconds', $Lifetime)->epoch,
        'set_iat'   => 1,
        'secret'    => $Secret // '',
        'claims'    => {
            'sub' => $Subject,
            'scope' => $Scope,
            'iss' => $Issuer,
            'aud' => $Auditor
        }
    });

    return $WebToken->encode;
}

1;
