#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::PerlTools qw(
    AddDuration
    CheckIpFormat
    Compare
    CopyStructure
    ElementOf
    GenerateGUID
    GenerateShortGUID
    GetCallerPackage
    GetIPAddress
    GetMD5Digest
    IndexOf
    Ip2Hex
    SubtractDuration
    CreateSecureCode
    VerifySecureCode
    fcmp
    trim
    unique
);
use Data::Dumper qw ( Dumper );
use DateTime;
use utf8;

plan tests => 14;

## no critic qw(ProhibitMagicNumbers)

subtest 'GetCallerPackage' => sub {
    ok( Test() eq 'main', 'GetCallerPackage main');
    ok( MyPackage::Test()  eq 'MyPackage',  'GetCallerPackage MyPackage');
    ok( MyPackage::Test2() eq 'MyPackage2',  'GetCallerPackage MyPackage2' );
};

# trim, unique
subtest 'trim_unique' => sub {
    ok( ('AAC'          eq join '', trim( '  A', ' A', "\t  C\n" )), 'trim' );
    ok( ("  A A\t  C\n" eq join '', unique( '  A', ' A', "\t  C\n" )), 'unique' );
    ok( ('AAC'          eq join '', trim unique( '  A', ' A', "\t  C\n" )), 'trim unique' );
    ok( ('AC'           eq join '', unique trim( '  A', ' A', "\t  C\n" )), 'unique trim' );
};

subtest 'CopyStructure' => sub {
    my $Structure1 = GetStructure();
    my $Structure2 = GetStructure();
    local $Data::Dumper::Sortkeys = 1;
    ok (Dumper($Structure1) eq Dumper(CopyStructure($Structure2)), 'structure equal');
};

subtest 'CheckIpFormat' => sub {
    ok( !defined CheckIpFormat('1.0.0.127'), 'CheckIpFormat IP4');
    ok( !defined CheckIpFormat('2001:4bd8:2:2:250:56ff:fe05:299'), 'CheckIpFormat IPv6');
    ok( !defined CheckIpFormat('::1'), 'CheckIpFormat localhost IPv6');
    cmp_ok( CheckIpFormat('100127'), 'eq', 'VOID_IP_ADDRESS', 'CheckIpFormat - no dots');
    cmp_ok( CheckIpFormat('x.0.0.127'), 'eq', 'VOID_IP_ADDRESS', 'CheckIpFormat - invalid characters');
    cmp_ok( CheckIpFormat('1.0..0.127'), 'eq', 'VOID_IPv4_ADDRESS', 'CheckIpFormat - invalid IPv4');
    cmp_ok( CheckIpFormat('1:1:2'), 'eq', 'VOID_IPv6_ADDRESS', 'CheckIpFormat - invalid IPv6');
};

subtest 'GetIPAddress' => sub {
    my $LocalIP = GetIPAddress();
    is( CheckIpFormat($LocalIP), undef, "GetIPAddress(): $LocalIP");

    my $IP = GetIPAddress('255.255.255.255', 4);
    is( $IP, '255.255.255.255', 'GetIPAddress(255.255.255.255)' );

    $IP = GetIPAddress('localhost', 4);
    is( $IP, '127.0.0.1', 'GetIPAddress(localhost)' );

    SKIP: {
        skip 'not testing external systems', 2 if $ENV{'SKIP_EXTERNAL_TESTS'};
        $IP = GetIPAddress('www.google.com');
        isnt( $IP, $LocalIP, 'GetIPAddress(www.google.com)' );
        is( CheckIpFormat($IP), undef, "CheckIpFormat($IP)" );
    }
};

subtest 'IP2Hex' => sub {
    cmp_ok( Ip2Hex('1.0.0.127'), 'eq', '0100007F', 'Ip2Hex localhost IP4');
    cmp_ok( Ip2Hex('::1'), 'eq', '00000000000000000000000000000001',  'Ip2Hex localhost IPv6');
    cmp_ok( Ip2Hex('2001:4bd8:2:2:250:56ff:fe05:299'), 'eq', '20014BD800020002025056FFFE050299', 'Ip2Hex IPv6');
    cmp_ok( Ip2Hex('2001:4bd8:2:2:250:56ff:fe05:299', 8), 'eq', 'FE050299', 'short Ip2Hex IPv6');
};

subtest 'GenerateGUID' => sub {
    my $GUID1 = GenerateGUID();
    ok ($GUID1 =~ /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$/x, 'GUID format'); ## no critic (RegularExpressions::ProhibitComplexRegexes)

    my $findDoubleGUID = 1;
    my %GUIDs;
    for(my $i=0;$i<100000;$i++) {
        my $GUID = GenerateGUID();
        $findDoubleGUID = 0 if exists $GUIDs{$GUID};
        $GUIDs{$GUID} = undef;
    }
    ok ($findDoubleGUID , '100000 created GUIDs not equal');
};

subtest 'GenerateShortGUID' => sub {
    my $ShortGUID = GenerateShortGUID();
    ok ($ShortGUID =~ /^[0-9A-F]{20}$/, 'GenerateShortGUID format');
};

subtest 'fcmp' => sub {
    ok( ( ElementOf('ten', qw(twelve eleven ten) )), 'ElementOf');
    ok( ( 2 == IndexOf('ten', qw(twelve eleven ten) )), 'IndexOf');
    ok( 0  == fcmp(0.00011,1.10 - 1.09989), 'fcmp real');
    ok( -1 == fcmp(0,1), 'fcmp(0,1)');
    ok( 0   == fcmp(0.00011,0.00012, 4), 'fcmp equal with 4 decimal digits');
    ok( -1  == fcmp(0.00011,0.00012, 5), 'fcmp less with 5 decimal digits');
};

subtest 'Compare' => sub {
    ok( 0 == Compare( 'String', undef, undef ), 'Compare undef to empty string' );
    ok( -1 == Compare( 'String', undef, '' ), 'Compare undef to empty string' );
    ok( 1 == Compare( 'String', '', undef ), 'Compare undef to empty string' );
    ok( 1 == Compare( 'String', 'a', 'B' ), 'Compare case sensitive' );
};

is( GetMD5Digest(\'test'), '098f6bcd4621d373cade4e832627b4f6', 'GetMD5Digest' );    #'

subtest 'CreateSecureCode' => sub {
    is( CreateSecureCode("m\x{F6}llerfranzmoellder\@epages.de"), '218aec30aca416ddc49e04b4f903227c', 'CreateSecureCode' );
    is( VerifySecureCode("m\x{F6}llerfranzmoellder\@epages.de", '218aec30aca416ddc49e04b4f903227c'), 1, 'VerifySecureCode - true' );
    is( VerifySecureCode("m\x{F6}llerfranzmoellder\@epages.de", 'cc9e0e7699711ccb4d9e077690076a52'), 0, 'VerifySecureCode - false' );
};

subtest 'AddDuration' => sub {
    my $dt = DateTime->new( year => 2010, month => 3, day => 28, hour => 0, minute => 30, time_zone => 'Europe/Berlin' );
    my $dt2 = AddDuration( $dt, 'hours', -2 );
    cmp_ok( "$dt",  'eq', '2010-03-28T00:30:00', 'original value has not changed' );
    cmp_ok( "$dt2", 'eq', '2010-03-27T22:30:00', 'substract 2 hours' );
    cmp_ok( $dt2->time_zone->name, 'eq', 'Europe/Berlin', 'time zone does not change' );

    my $dt3 = AddDuration( $dt, 'hours', 2 );
    cmp_ok( "$dt3", 'eq', '2010-03-28T03:30:00', 'skip invalid hour during daylight saving time switch' );

    $dt = DateTime->new( year => 2010, month => 3, day => 28, hour => 0, minute => 30 );
    my $dt4 = AddDuration( $dt, 'hours', 2 );
    cmp_ok( "$dt4", 'eq', '2010-03-28T02:30:00', 'add 2 hours to floating time' );
    cmp_ok( $dt4->time_zone->name, 'eq', 'floating', 'floating time zone does not change' );
};

subtest 'SubtractDuration' => sub {
    my $dt = DateTime->new( year => 2010, month => 3, day => 28, hour => 0, minute => 30 );
    my $dt5 = SubtractDuration( $dt, 'hours', 2 );
    cmp_ok( "$dt5", 'eq', '2010-03-27T22:30:00', 'substract 2 hours from floating time' );
    cmp_ok( $dt, 'eq', $dt5, 'inpupt parameter has been modified' );
    cmp_ok( $dt5->time_zone->name, 'eq', 'floating', 'floating time zone does not change' );
};

# -----------------------------------------------------------------------------
# helper functions

sub Test {
    my $MyPackage = GetCallerPackage();
    return $MyPackage;
}
sub GetStructure {
    return {
        'Test1' => [1,3,4,5,6],
        'Test2' => {
            'AA' => 2,
            'BB' => 5,
        },
    };
}

##no critic (ProhibitMultiplePackages RequireCorrectImports)
package MyPackage; ## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)
use DE_EPAGES::Core::API::PerlTools qw( GetCallerPackage );
sub Test {
    my $MyPackage = GetCallerPackage();
    return $MyPackage;
}
sub Test2 { return MyPackage2::Test(); }

package MyPackage2;
use DE_EPAGES::Core::API::PerlTools qw( GetCallerPackage );
sub Test {
    my $MyPackage = GetCallerPackage();
    return $MyPackage;
}
##use critic

