#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse
# state        public
#----------------------------------------------------------------------------------------
# description  Response parser for getPackstationsByAddress DHL Intraship webservice method
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse;

use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw ( $NUMBER_OF_PACKSTATIONS );
use Math::Round qw ( nearest );

use strict;

use constant METERS_TO_KMS => 1000;

#ERRORDEFINITION
sub ErrorDefinition {
    return { 'DHL_RESPONSE_FAULT' => 'Code: #FaultCode . Id: #FaultId . Reason: #FaultReason' };
}

#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse->new();
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# return       $self | ReturnDescription | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     parseResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       parseResponse( $SOAPResponse );
#----------------------------------------------------------------------------------------
# description  Parse the response from the getPackstationsByAddress
#----------------------------------------------------------------------------------------
# input        $SOAPResponse | Hash response from DHL Intraship location method | ref_hash
# return       \@Packstations | List of packstations | refarray
#========================================================================================
sub parseResponse {
    my $self = shift;
    my ($SOAPResponse) = @_;

    my @Packstations;
    if ( $SOAPResponse->{'Fault'} ) {
        my $faultCode = $SOAPResponse->{'Fault'}->{'faultcode'} // 'Fault';
        my $faultReason = $SOAPResponse->{'Fault'}->{'faultstring'} // 'Fault';
        my $faultId = 'No-Id';

        if ( $SOAPResponse->{'ServiceException'} ) {
            $faultId = $SOAPResponse->{'ServiceException'}->{'id'} // 'No Service Exception Id provided';
        }

        $self->_Warning( 'DHL_RESPONSE_FAULT', {
                    'FaultCode' => $faultCode,
                    'FaultId' => $faultId,
                    'FaultReason' => $faultReason,
                }
        );
    } else {
        my $PackstationsList = $SOAPResponse->{'parameters'}->{'packstation'};

        foreach my $Packstation (@$PackstationsList) {
            push(@Packstations, $self->_getAddress($Packstation));
            if ( scalar(@Packstations) == $NUMBER_OF_PACKSTATIONS ) {
                last;
            }
        }
    }

    return \@Packstations;
}

#----------------------------------------------------------------------------------------
sub _getAddress {
    my $self = shift;
    my ($Packstation) = @_;

    return unless isValid($Packstation);

    my $DistanceInKms = $self->_convert2Kms($Packstation->{'location'}->{'distance'});

    my $hAddress = {
        'name'          => $Packstation->{'address'}->{'remark'},
        'address'       => $Packstation->{'address'}->{'street'} . ' ' . $Packstation->{'address'}->{'streetNo'},
        'zipcode'       => $Packstation->{'address'}->{'zip'},
        'city'          => $Packstation->{'address'}->{'city'},
        'district'      => $Packstation->{'address'}->{'district'},
        'distance'      => $DistanceInKms,
        'latitude'      => $Packstation->{'location'}->{'latitude'}->bstr(),
        'longitude'     => $Packstation->{'location'}->{'longitude'}->bstr(),
        'packstationId' => $Packstation->{'packstationId'},
    };

    return $hAddress;
}

#----------------------------------------------------------------------------------------
sub _convert2Kms {
    my $self = shift;
    my ($Distance) = @_;

    if ($Distance >= METERS_TO_KMS) {
        ## no critic (ProhibitMagicNumbers)
        $Distance = nearest(.1, $Distance / METERS_TO_KMS) . ' km';
        ## use critic (ProhibitMagicNumbers)
    } else {
        $Distance = $Distance . ' m';
    }

    return $Distance;
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _Warning {
    my $self = shift;
    my ($WarningTitle, $hWarning) = @_;

    Warning($WarningTitle, $hWarning);

    return;
}

1;