#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder
# state        public
# description  DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentOrder;
use DE_EPAGES::DHLIntraship::API::Constants qw ( $DHLVERSENDEN_SHIPMENT_RELEASE );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );

use strict;

#========================================================================================
sub new {
    my $class = shift;

    my $self = bless { 'createShipmentOrder' => \&_createShipmentOrder, }, $class;
    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       assemble( $operation, $aShippingLabels );
#----------------------------------------------------------------------------------------
# description  Create the hash with all necessary values for the request
#----------------------------------------------------------------------------------------
# input        $operation | Operation name | String
# input        $ShippingLabel | Shipping Label object | object
# return       $ReturnVariable | ReturnDescription | refhash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($operation, $ShippingLabel) = @_;

    return unless (defined ($self->{$operation}));
    my $ReturnVariable = $self->{$operation}($self, $ShippingLabel);

    return $ReturnVariable;
}

#========================================================================================
# function     _createShipmentOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hShippingLabelRequest = $self->_createShipmentOrder($ShippingLabel);
#----------------------------------------------------------------------------------------
# description  Create the hash with all necessary values for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL intraship shipping label object | object
# return       $hShippingLabelRequest | DHL Versenden shipping label service request | ref.hash
#========================================================================================
sub _createShipmentOrder {
    my $self = shift;
    my ($ShippingLabel) = @_;

    TestObject( 'ShippingLabelDHLIntraship', $ShippingLabel,
        'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' );

    my $hShipmentOrder = $self->_assembleShipmentOrder($ShippingLabel);

    $self->_LogShippingFulfillment('DHLIntraship', 'Builder Warnings: ', $hShipmentOrder->{'nodeWarnings'})
        if (scalar(@{$hShipmentOrder->{'nodeWarnings'}}) > 0);

    return {
        %{ $self->_assembleAuthentification($ShippingLabel) },
        'part1' => {
                 %{ $self->_assembleVersion() },
                 %{ $hShipmentOrder->{'nodeData'} },
        }
    };
}

#========================================================================================
# function     _assembleAuthentification
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAuthentification = $self->_assembleAuthentification($ShippingLabel);
#----------------------------------------------------------------------------------------
# description  Create the hash with authentification data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL intraship shipping label object | object
# return       $hAuthentification | DHL Versenden authentification | ref.hash
#========================================================================================
sub _assembleAuthentification {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $ShippingFulfillmentMethod = $ShippingLabel->get('ShippingFulfillmentMethod');
    TestObject( 'ShippingFulfillmentMethodDHLIntraship', $ShippingFulfillmentMethod,
        'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship' );

    return {
        'header' => {
            'user'      => $ShippingFulfillmentMethod->get('WebserviceUsername'),
            'signature' => $ShippingFulfillmentMethod->get('WebservicePassword'),
        }
    };
}

#========================================================================================
# function     _assembleVersion
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hVersion = $self->_assembleVersion();
#----------------------------------------------------------------------------------------
# description  Create the hash with version data for the Shipping label request
#----------------------------------------------------------------------------------------
# return       $hVersion | DHL Versenden shipping version | ref.hash
#========================================================================================
sub _assembleVersion {
    my $self = shift;

    my @VersionNumbers = ( $DHLVERSENDEN_SHIPMENT_RELEASE =~ /\d+/g );
    return {
        'Version' => {
            'majorRelease' => $VersionNumbers[0],
            'minorRelease' => $VersionNumbers[1],
        }
    };
}

#========================================================================================
# function     _assembleShipmentOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hShipmentOrder = $self->_assembleShipmentOrder($ShippingLabel);
#----------------------------------------------------------------------------------------
# description  Create the hash with shipment order data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hShipmentOrder | DHL Versenden shipment order | ref.hash
#========================================================================================
sub _assembleShipmentOrder {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::ShipmentOrder->assemble($ShippingLabel);
}

#========================================================================================
# wrappers
#========================================================================================
sub _LogShippingFulfillment {
    my $self = shift;
    my ($ShippingFulfillmentTypeAlias, $Name, $DebugStuff) = @_;

    LogShippingFulfillment($ShippingFulfillmentTypeAlias, $Name, $DebugStuff);
}

1;
