#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient
# state        public
# description  Description
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;

use strict;
use Data::Dumper qw ( Dumper );

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::DHLIntraship::API::BuildEndPoint;
use DE_EPAGES::DHLIntraship::API::BuildLoginData;
use DE_EPAGES::DHLIntraship::API::Constants qw (
        $LOCATION_WSDL
        $SHIPPING_WSDL
        @PORTS
        $DHLINTRASHIP_PATH_WSDL
        $XSD_STANDORTE_FILE
        $XSD_CISBASE_FILE
        $XSD_ISBASE_FILE
        $GET_MANIFEST_OPERATION_IS_UNKNOWN
);
use DE_EPAGES::DHLIntraship::API::DeveloperAuthentication;
use DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer;
use DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client;
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use XML::Compile::SOAP11;
use XML::Compile::Transport::SOAPHTTP;
use XML::Compile::WSDL11;


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new($Builder, $Parser, $IsTest, $hLoginData);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Builder    | Builder package        | String
# input        $Parser     | Parser package         | String
# input        $IsTest     | Whether is test or not | boolean
# input        $hLoginData | Login data             | hash
# return       $self | self | object
#========================================================================================
sub new {
    my $class = shift;
    my ($Builder, $Parser, $IsTest, $hLoginData) = @_;
    my $self  = bless {
    }, $class;

    $self->{'Builder'} = $Builder;
    $self->{'Parser'} = $Parser;
    $self->{'hLoginData'} = (isValid($hLoginData)) ? $hLoginData : $self->_getLoginData($IsTest);
    $self->{'IsTest'} = $IsTest;

    return $self;
}

#========================================================================================
# function     setSOAPOperationName
# state        public
#----------------------------------------------------------------------------------------
# syntax       setSOAPOperationName('createShipmentOrder');
#----------------------------------------------------------------------------------------
# description  sets the SOAP operation name.
#----------------------------------------------------------------------------------------
# input        $Input1 | Name of the SOAP operation | String
#========================================================================================
sub setSOAPOperationName {
    my $self = shift;
    my ($SOAPOperationName) = @_;
    $self->{'OperationName'} = $SOAPOperationName;
    return;
}


#========================================================================================
# function     createShipmentOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       createShipmentOrder( $ShippingLabel );
#----------------------------------------------------------------------------------------
# description  execute the createShipmentOrder SOAP request
#               throws HttpError if result is empty.
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | Shipping label Object | Object
# return       $aResponse | Simplified response array | refarray
#========================================================================================
sub createShipmentOrder {
    my $self = shift;
    my ($ShippingLabel) = @_;

    TestObject('ShippingLabelDHLIntraship', $ShippingLabel, 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship');
    $self->setSOAPOperationName('createShipmentOrder');
    my ($aResponse, $Trace) = $self->_executeCall( $ShippingLabel );

    # If Http Status is not ok (status codes 4xx, 5xx), the result is empty
    my $ResponseSize = @{$aResponse};
    if ( $ResponseSize == 1 && @$aResponse[0]->{'IsError'} ) {
        $self->_LogDataWithoutPasswords($self->{'OperationName'}.' Trace: ', $Trace);
    }

    return $aResponse;
}

#========================================================================================
# function     getPackstationsByAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPackstationsByAddress( $ShippingFulfillmentMethod );
#----------------------------------------------------------------------------------------
# description  Retrieve the list of packstations from DHL Intraship
#----------------------------------------------------------------------------------------
# input        $ShippingFulfillmentMethod | ShipingFulfillmentMethod object | object
# return       $aPackstations | List of packstations | refarray
#========================================================================================
sub getPackstationsByAddress {
    my $self = shift;
    my ($Servlet) = @_;

    $self->setSOAPOperationName('getPackstationsByAddress');
    my ($aPackstations) = $self->_executeCall( $Servlet );

    return $aPackstations;
}

#========================================================================================
# function     getManifest
# state        public
#----------------------------------------------------------------------------------------
# syntax       getManifest( $hInputParameters );
#----------------------------------------------------------------------------------------
# description  Retrieve manifest report for an specified date
#----------------------------------------------------------------------------------------
# input        $hInputParameters | hash with credentials and date | ref.hash
# return       $aResponse | Simplified response array | refarray
#========================================================================================
sub getManifest {
    my $self = shift;
    my ($hInputParameters) = @_;

    $self->setSOAPOperationName('getManifest');
    my ($hResponse, $Trace) = $self->_executeCall( $hInputParameters );

    if ($hResponse->{'login'} eq $GET_MANIFEST_OPERATION_IS_UNKNOWN) {
        $self->_LogDataWithoutPasswords($self->{'OperationName'}.' Trace: ', $Trace);
    }

    return $hResponse;
}

#========================================================================================
# function     _executeCall
# state        private
#----------------------------------------------------------------------------------------
# syntax       _executeCall( $RequestObject );
#----------------------------------------------------------------------------------------
# description  Assembles WebService calls, sends requests and parses responses.
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | Shipping label Object | Object
# return       $aResponse | response array | refarray
# return       $Trace | request trace hash or undef | object
#========================================================================================
sub _executeCall {
    my $self = shift;
    my ( $RequestObject ) = @_;

    my $RequestData = $self->{'Builder'}->assemble($self->{'OperationName'}, $RequestObject);
    $self->_LogDataWithoutPasswords($self->{'OperationName'}.' Request: ', $RequestData);

    my ($Response, $Trace) = $self->_sendRequest( $RequestData );

    if (defined($Response)) {
        $self->_LogDataWithoutPasswords($self->{'OperationName'}.' Response: ', $Response);
    } else {
        $self->_LogDataWithoutPasswords($self->{'OperationName'}.' Trace: ', $Trace);
    }

    my $aResponse  = $self->{'Parser'}->parseResponse($Response);

    return ($aResponse, $Trace);
}

#========================================================================================
# function     createTransportLayer
# state        public
#----------------------------------------------------------------------------------------
# syntax       createTransportLayer( );
#----------------------------------------------------------------------------------------
# description  Creates Transport Layer
#----------------------------------------------------------------------------------------
# return       $TransportLayer | Transport Layer | XML::Compile::Transport::SOAPHTTP
#========================================================================================
sub createTransportLayer {
    my $self = shift;

   my $TransportLayerBuilder =
        DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer->new(
            $self->{'OperationName'},
            $self->{'hLoginData'}
        )
        ->withSSLPort();
    return $TransportLayerBuilder->getTransportLayer();
}


#========================================================================================
# function     _sendRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _sendRequest( $RequestData );
#----------------------------------------------------------------------------------------
# description  handle the request to DHL Intraship
#----------------------------------------------------------------------------------------
# input        $RequestData | the Data which will be sended | refhash
# return       $Response | respose of the SOAP request | Object
# return       $Trace | request trace hash | refhash
#========================================================================================
sub _sendRequest {
    my $self = shift;
    my ( $RequestData ) = @_;

    my $CompiledSOAPClient = $self->_getCompiledSOAPClient();

    my ( $Response, $Trace ) = $CompiledSOAPClient->($RequestData);

    return ( $Response, $Trace );
}

#========================================================================================
# function     _getCompiledSOAPClient
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getCompiledSOAPClient( );
#----------------------------------------------------------------------------------------
# description  takes all DHLIntraship data and compile a WSDL11 client
#----------------------------------------------------------------------------------------
# return       $CompiledSOAPClient | ready comliled WSDL11 client | object
#========================================================================================
sub _getCompiledSOAPClient {
    my $self = shift;

    my $Folder      = $ENV{'EPAGES_CARTRIDGES'} . $DHLINTRASHIP_PATH_WSDL;
    my $aXSDFiles = [$Folder . $XSD_CISBASE_FILE, $Folder . $XSD_ISBASE_FILE];

    my $WSDLPath = $Folder . $SHIPPING_WSDL;
    if ( $self->{'OperationName'} eq 'getPackstationsByAddress' ) {
        $WSDLPath = $Folder . $LOCATION_WSDL;
        $aXSDFiles = [$Folder . $XSD_STANDORTE_FILE];
    }

    my $TransportLayer = $self->createTransportLayer();
    ## no critic (ValuesAndExpressions::ProhibitLongChainsOfMethodCalls)
    my $WSDL11Client = DE_EPAGES::DHLIntraship::API::SOAP::BuildWSDL11Client->new($self->{'OperationName'}, $self->{'IsTest'})
        ->withWSDLFile($WSDLPath)
        ->withXSDFiles($aXSDFiles)
        ->withTransportLayer($TransportLayer)
    ;
    ## use critic (ValuesAndExpressions::ProhibitLongChainsOfMethodCalls)

    my $CompiledSOAPClient = $WSDL11Client->compile();
    return $CompiledSOAPClient;
}
#========================================================================================
# function     _LogDataWithoutPasswords
# state        private
#----------------------------------------------------------------------------------------
# syntax       _LogDataWithoutPasswords( $SOAP_Operation, $Trace );
#----------------------------------------------------------------------------------------
# description  Encapsulate the LogShippingFulfillment for replacing the passwords
#----------------------------------------------------------------------------------------
# input        $SOAP_Operation | name of the SOAP operation | String
# input        $TextLog | Text wich should be logged | String
#========================================================================================
sub _LogDataWithoutPasswords{
    my $self = shift;
    my ($SOAP_Operation, $TextLog) = @_;

    # Epages specific Log dumper settings don't print the whole structure for shipment orders requests and responses.
    if ($SOAP_Operation ~~ /^createShipmentOrder/) {
        $TextLog = Dumper($TextLog);
    }
    LogShippingFulfillment('DHLIntraship', $SOAP_Operation, $TextLog, undef,
            sub{
                my ($hashString) = @_;
                ${$hashString} =~ s/(?<=<cis:signature>).*?(?=<\/cis:signature>)/---replaced---/msxig;
                ${$hashString} =~ s/(?<='signature'\s=>\s').*?(?=',)/---replaced---/msxig;
                ${$hashString} =~ s/(?<=\\'signature\\'\s=>\s\\').*?(?=\\',)/---replaced---/msxig;
            }
    );

    return;
}

sub _getLoginData {
    my $self = shift;
    my ($IsTest) = @_;

    my $EndPoint = DE_EPAGES::DHLIntraship::API::BuildEndPoint->new($IsTest);
    my $DeveloperAuthentication = DE_EPAGES::DHLIntraship::API::DeveloperAuthentication->new($IsTest);

    my $BuildLoginData = DE_EPAGES::DHLIntraship::API::BuildLoginData->new()
        ->withEndPoint($EndPoint)
        ->withDeveloperAuthentication($DeveloperAuthentication);

    return $BuildLoginData->getAsHash();
}

1;