#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::WebServiceClient
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for WebServiceClient.
#========================================================================================
## no critic (Tics::ProhibitLongLines)
package DE_EPAGES::DHLIntraship::t::unit::WebServiceClient;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw ( $STATUS_CODE_OK );
use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::More;
use XML::Compile::Transport::SOAPHTTP;

use strict;

use constant TEST_LOGIN_DATA => {
        'UserName' => 'TestUser',
        'Password' => 'TestPassword',
        'Realm' => 'TestRealm',
        'EndPointURL' => 'https://www.test.example',
};

use constant IS_TEST => 1;

sub test {
    my $self = shift;
    $self->_test_createShipmentOrder();
    $self->_test_createShipmentOrder_missing_ShippingLabel();
    $self->_test_getPackstationsByAddress();
    $self->_test_getManifest();
    $self->_test_createTransportLayer_ShippingOrder();
    $self->_test_createTransportLayer_Packstation();
    done_testing();
}

#----------------------------------------------------------------------------------------

sub _test_createShipmentOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedWebServiceClient = $self->_buildWebServiceClient_createShipmentOrder();
    my $MockedShippingLabel = $self->_createShippingLabel();
    my $aResponse = $MockedWebServiceClient->createShipmentOrder($MockedShippingLabel);
    my $StatusCode = @$aResponse[0]->{'Status'}->{'statusCode'};

    is($StatusCode, $STATUS_CODE_OK, "$SubTestName - Status Code is OK.");
    is($MockedWebServiceClient->{'hLoginData'}->{'UserName'}, TEST_LOGIN_DATA->{'UserName'},
            "$SubTestName - Login Data has been correctly set.");
    isa_ok($MockedWebServiceClient->{'Builder'}, 'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder',
            "$SubTestName - Builder is the right one.");
    isa_ok($MockedWebServiceClient->{'Parser'}, 'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser',
            "$SubTestName - Builder is the right one.");
    my $aParameters = GetParametersFromMockifyCall($MockedWebServiceClient, 'setSOAPOperationName');
    is_deeply($aParameters, ['createShipmentOrder'], "$SubTestName - Operation name was correctly set.");

    return;
}

sub _test_createShipmentOrder_missing_ShippingLabel {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedWebServiceClient = $self->_buildWebServiceClient_createShipmentOrder();
    throws_ok {$MockedWebServiceClient->createShipmentOrder()} qr/PARAMETER_MISSING/,
            "$SubTestName - Shipping label is missing.";

    return;
}

sub _test_getPackstationsByAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedWebServiceClient = $self->_buildWebServiceClient_getPackstationsByAddress();
    my $MockedServlet = $self->_createServlet();
    my $hPackstations = $MockedWebServiceClient->getPackstationsByAddress($MockedServlet);
    ok(isValid($hPackstations->{'parameters'}->{'packstation'}), "$SubTestName - Response has been correctly parsed.");
    is($MockedWebServiceClient->{'hLoginData'}->{'UserName'}, TEST_LOGIN_DATA->{'UserName'},
            "$SubTestName - Login Data has been correctly set.");
    isa_ok($MockedWebServiceClient->{'Builder'}, 'DE_EPAGES::DHLIntraship::API::SOAP::PackstationRequest',
            "$SubTestName - Builder is the right one.");
    isa_ok($MockedWebServiceClient->{'Parser'}, 'DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse',
            "$SubTestName - Builder is the right one.");
    my $aParameters = GetParametersFromMockifyCall($MockedWebServiceClient, 'setSOAPOperationName');
    is_deeply($aParameters, ['getPackstationsByAddress'], "$SubTestName - Operation name was correctly set.");

    return;
}

sub _test_getManifest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedWebServiceClient = $self->_buildWebServiceClient_getManifest();
    my $TestCredentials = {
        'user' => 'userToTest',
        'signature' => 'signatureToTest'
    };
    my $hManifestData = $MockedWebServiceClient->getManifest($TestCredentials);
    ok(isValid($hManifestData->{'Status'}), "$SubTestName - Response has been correctly parsed.");
    is($MockedWebServiceClient->{'hLoginData'}->{'UserName'}, TEST_LOGIN_DATA->{'UserName'},
            "$SubTestName - Login Data has been correctly set.");
    isa_ok($MockedWebServiceClient->{'Builder'}, 'DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest',
            "$SubTestName - Builder is the right one.");
    isa_ok($MockedWebServiceClient->{'Parser'}, 'DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationResponse',
            "$SubTestName - Builder is the right one.");
    my $aParameters = GetParametersFromMockifyCall($MockedWebServiceClient, 'setSOAPOperationName');
    is_deeply($aParameters, ['getManifest'], "$SubTestName - Operation name was correctly set.");
}

sub _test_createTransportLayer_ShippingOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedWebServiceClient = $self->_buildWebServiceClient_createShipmentOrder();
    my $TransportLayer = $MockedWebServiceClient->createTransportLayer();
    isa_ok($TransportLayer, 'XML::Compile::Transport::SOAPHTTP', "$SubTestName - Transport layer is okay.");

    return;
}

sub _test_createTransportLayer_Packstation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedWebServiceClient = $self->_buildWebServiceClient_getPackstationsByAddress();
    my $TransportLayer = $MockedWebServiceClient->createTransportLayer();
    isa_ok($TransportLayer, 'XML::Compile::Transport::SOAPHTTP', "$SubTestName - Transport layer is okay.");

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _buildWebServiceClient_createShipmentOrder {
    my $self = shift;
    my $aAttributes = [
        $self->_createShippingLabelRequestBuilder(),
        $self->_createShippingLabelResponseParser(),
        IS_TEST,
        TEST_LOGIN_DATA
    ];

    my $hReturnValues = {
        'WebServiceResponse' => {
            'part1' => {}
        },
        '_getLoginData' => TEST_LOGIN_DATA,
        'createTransportLayer' => $self->_createBuildTransportLayer(),
    };

    return $self->_createWebServiceClient($hReturnValues, $aAttributes);
}

sub _buildWebServiceClient_getPackstationsByAddress {
    my $self = shift;

    my $aAttributes = [
        $self->_createPackstationRequest(),
        $self->_createPackstationResponse(),
        IS_TEST,
        TEST_LOGIN_DATA
    ];

    my $hReturnValues = {
        'WebServiceResponse' => {
            'parameters' => {
                'packstation' => []
            }
        },
        '_getLoginData' => TEST_LOGIN_DATA,
        'createTransportLayer' => $self->_createBuildTransportLayer(),
    };

    return $self->_createWebServiceClient($hReturnValues, $aAttributes);
}

sub _buildWebServiceClient_getManifest {
    my $self = shift;

    my $aAttributes = [
        $self->_createCredentialsValidationRequest(),
        $self->_createCredentialsValidationResponse(),
        IS_TEST,
        TEST_LOGIN_DATA
    ];

    my $hReturnValues = {
        'WebServiceResponse' => {
            'part1' => {
                'Status' => {}
            }
        },
        '_getLoginData' => TEST_LOGIN_DATA,
        'createTransportLayer' => $self->_createBuildTransportLayer(),
    };

    return $self->_createWebServiceClient($hReturnValues, $aAttributes);
}

sub _createWebServiceClient {
    my $self = shift;
    my ($hReturnValues, $aAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient', $aAttributes
    );

    $Mockify->addMock('_sendRequest', sub {
        shift;

        return $hReturnValues->{'WebServiceResponse'};
    });
    $Mockify->addMockWithReturnValue('createTransportLayer', $hReturnValues->{'createTransportLayer'});
    $Mockify->addMethodSpyWithParameterCheck('setSOAPOperationName', ['string']);
    $Mockify->addMockWithReturnValueAndParameterCheck('_LogDataWithoutPasswords', undef, [ 'string', 'any' ]);

    return $Mockify->getMockObject();
}

sub _createShippingLabel {
    my $self = shift;

    my $MockifyShippingLabel = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship', {}
    );

    return $MockifyShippingLabel->getMockObject();
}

sub _createServlet {
    my $self = shift;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($self->_createShippingFulfillmentMethod());
    $MockifyServlet->addParams(
            {
                'Street' => 'Heinrich-Heine-Str.',
                'StreetNo' => '1',
                'ZipCode' => '07749',
                'City' => 'Jena'
            }
    );
    return $MockifyServlet->getMockObject();
}

sub _createShippingFulfillmentMethod {
    my $self = shift;

    my $MockifyShippingFulfillmentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship', {}
    );

    return $MockifyShippingFulfillmentMethod->getMockObject();
}

sub _createBuildTransportLayer {
    my $self = shift;

    return XML::Compile::Transport::SOAPHTTP->new();
}

sub _createShippingLabelRequestBuilder {
    my $self = shift;

    my $aAttributes = [];
    my $ShippingLabelRequestBuilderMockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder', $aAttributes
    );

    my $hBuiltRequest = {
        'header' => {
            'signature' => 'test_password',
            'user' => 'test_user'
        },
        'part1' => {
            'ShipmentOrder' => [{
                'Shipment' => {
                    'Receiver' => {},
                    'ShipmentDetails' => {},
                    'Shipper' => {}
                },
                'labelResponseType' => 'URL',
                'sequenceNumber' => '21548'
            }],
            'Version' => {
                'majorRelease' => '2',
                'minorRelease' => '2'
            }
        }
    };
    $ShippingLabelRequestBuilderMockify->addMockWithReturnValueAndParameterCheck('assemble', $hBuiltRequest,
            [ 'string', { 'object' => 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' } ]);

    return $ShippingLabelRequestBuilderMockify->getMockObject();
}

sub _createShippingLabelResponseParser {
    my $self = shift;

    my $aAttributes = [];
    my $ShippingLabelResponseParserMockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser', $aAttributes
    );
    my $aParsedResponse = [{
        'CreationState' => [{
            'LabelData' => {
                'Status' => {},
                'labelUrl' => 'https://cig.dhl.de/...',
                'shipmentNumber' => '222201010031594518'
            },
            'sequenceNumber' => '21548'
        }],
        'Status' => {
            'statusCode' => '0',
            'statusMessage' => ['Der Webservice wurde ohne Fehler ausgefhrt.'],
            'statusText' => 'ok'
        },
        'Version' => {
            'majorRelease' => '2',
            'minorRelease' => '0'
        }
    }];
    $ShippingLabelResponseParserMockify->addMockWithReturnValueAndParameterCheck(
            'parseResponse', $aParsedResponse, [ 'hashref' ]
    );

    return $ShippingLabelResponseParserMockify->getMockObject();
}

sub _createPackstationRequest {
    my $self = shift;

    my $aAttributes = [];
    my $PackstationRequestMockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::PackstationRequest', $aAttributes
    );
    my $hBuiltRequest = {
        'address' => {
            'city' => 'Jena',
            'countryCode' => 'DE',
            'street' => 'Heinrich-Heine-Str.',
            'streetNo' => '1',
            'zip' => '07749'
        },
        'key' => ''
    };
    $PackstationRequestMockify->addMockWithReturnValueAndParameterCheck('assemble', $hBuiltRequest,
            [ 'string', { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' } ]);

    return $PackstationRequestMockify->getMockObject();
}

sub _createPackstationResponse {
    my $self = shift;

    my $aAttributes = [];
    my $PackstationResponseMockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse', $aAttributes
    );
    my $hParsedResponse = {
        'parameters' => {
            'packstation' => [
                {
                    'address' => {},
                    'automatType' => 4,
                    'distance' => '1473',
                    'hasMobileParcelStampPrintService' => 0,
                    'hasXLPostfach' => 1,
                    'id' => '407743101',
                    'location' => {},
                    'objectId' => '7743101',
                    'opStatusType' => 1,
                    'packstationId' => '101',
                    'positionType' => 0,
                    'timeinfos' => {}
                },
                {
                    'address' => {},
                    'automatType' => 4,
                    'distance' => '1563',
                    'hasMobileParcelStampPrintService' => 0,
                    'hasXLPostfach' => 0,
                    'id' => '407743103',
                    'location' => {},
                    'objectId' => '7743103',
                    'opStatusType' => 1,
                    'packstationId' => '103',
                    'positionType' => 0,
                    'timeinfos' => {}
                }
            ]
        }
    };
    $PackstationResponseMockify->addMockWithReturnValueAndParameterCheck(
            'parseResponse', $hParsedResponse, [ 'hashref' ]
    );

    return $PackstationResponseMockify->getMockObject();
}

sub _createCredentialsValidationRequest {
    my $self = shift;

    my $aAttributes = [];
    my $CredentialsValidationRequestMockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest', $aAttributes
    );
    my $hBuiltRequest = {
        'header' => {},
        'part1' => {
            'Version' => {},
            'manifestDate' => 'string_date'
        }
    };
    $CredentialsValidationRequestMockify->addMockWithReturnValueAndParameterCheck('assemble', $hBuiltRequest,
            [ 'string', 'hashref' ]);

    return $CredentialsValidationRequestMockify->getMockObject();
}

sub _createCredentialsValidationResponse {
    my $self = shift;

    my $aAttributes = [];
    my $CredentialsValidationResponseMockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationResponse', $aAttributes
    );
    my $hParsedResponse = {
        'Status' => {}
    };
    $CredentialsValidationResponseMockify->addMockWithReturnValueAndParameterCheck(
            'parseResponse', $hParsedResponse, [ 'hashref' ]
    );

    return $CredentialsValidationResponseMockify->getMockObject();
}

__PACKAGE__->runScript();

1;