#========================================================================================
# package      DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for ShippingFulfillmentMethod.
#========================================================================================
package DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::ExternalShipping::API::Shop qw ( GetCurrentShippingOrder );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShippingFulfillmentMethod->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShippingFulfillmentMethod' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShippingFulfillmentMethod->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod = DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShippingFulfillmentMethod');
#               my $ShippingFulfillmentMethod = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShippingFulfillmentMethod' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingFulfillmentMethod | new object | object
# hook         OBJ_InsertShippingFulfillmentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodID - primary key          - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShippingFulfillmentMethod', {'Object' => $self, 'ShippingFulfillmentMethodID' => $self->id });
    $self->_refreshCache();
    delete $hInfo->{$_} foreach qw ( ShippingFulfillmentType IsInTestMode WebserviceUsername WebservicePassword AutoGenerateLabels Shop ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShippingFulfillmentMethod' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingFulfillmentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodID - primary key          - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShippingFulfillmentMethod', {'Object' => $self, 'ShippingFulfillmentMethodID' => $self->id });
    $self->_refreshCache();
    LoadClassByAlias('ShippingLabel')->deleteObjects('ByShippingFulfillmentMethodID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShippingFulfillmentMethod' and 'OBJ_AfterUpdateShippingFulfillmentMethod'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShippingFulfillmentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodID - primary key          - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShippingFulfillmentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentMethodID - primary key          - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShippingFulfillmentMethod', {'Object'=>$self, 'Values'=> $hValues, 'ShippingFulfillmentMethodID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingFulfillmentMethod',  {'Object'=>$self, 'Values'=> $hValues, 'ShippingFulfillmentMethodID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsInTestMode' => 0,
        'AutoGenerateLabels' => 0,
        'Shop' => $Parent->getSite(),
    };
}

#========================================================================================
# function     isUsable
# state        public
#----------------------------------------------------------------------------------------
# syntax       isUsable( );
#----------------------------------------------------------------------------------------
# description  Checks if ShippingFulfillmentMethod is usable
#----------------------------------------------------------------------------------------
# return       $IsUsable | True if ShippingFulfillmentMethod is usable | boolean
#========================================================================================
sub isUsable {
    my $self = shift;

    return 1;
}

#========================================================================================
# function     newShippingLabel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FulfillmentMethod->newShippingLabel( $Order );
# syntax       $FulfillmentMethod->newShippingLabel( $OrderDocument );
#----------------------------------------------------------------------------------------
# description  Creates a new shipping label, using the class
#               ShippingType.ShippingLabelClass.
#               This label is saved to the folder 'ShippingLabels'
#               Either an order or an order document can be provided as template to clone
#               from.
#----------------------------------------------------------------------------------------
# input        $Order | customer order | object
#               $OrderDocument | order document | object
# return       $ShippingLabel | shipping label | object
#========================================================================================
sub newShippingLabel {
    my $self = shift;
    my ($OrderOrDocument) = @_;
    TestObject('OrderOrDocument',$OrderOrDocument);

    my $Order;
    my $LineItemContainer;
    if($OrderOrDocument->instanceOf('OrderDocument')) {
        $Order = $OrderOrDocument->get('Order');
        $LineItemContainer = $OrderOrDocument->get('LineItemContainer');
    } elsif ($OrderOrDocument->instanceOf('CustomerOrder')) {
        $Order = $OrderOrDocument;
        $LineItemContainer = $Order->get('LineItemContainer');
    }

    my $Type          = $self->get('ShippingFulfillmentType');
    my $LabelClass    = $Type->get('ShippingLabelClass');

    my $Folder = $Order->folder('ShippingLabels');
    my $Label = $Folder->insertChild($LabelClass, {
        'Alias' => $Folder->nextAlias($Order->alias.'-%d'),
        'Order' => $Order,
        'ShippingFulfillmentMethod' => $self,
        'IsTestLabel' => $self->get('IsInTestMode'),
        'ShippingTransaction' => GetCurrentShippingOrder($self->getSite(), $Type),
        'WeightUnit' => $self->getWeightUnit(),
        'SizeUnit'   => $self->getSizeUnit(),
    });

    # clone the LineItemContainer to the shipping Label
    my $LineItemContainerClone = $LineItemContainer->clone({
        'Parent'  => $Label,
        'Alias'   => 'LineItemContainer',
        'Inherit' => 1,
    },0);

    # clone lineitemproducts without deposit
    foreach my $LineItem (@{$LineItemContainer->get('LineItems')}) {
        next unless ($self->isLineItemSupported($LineItem));
        $LineItem->clone({
            'Parent' => $LineItemContainerClone,
            'LineItemDeposit' => undef,
        },1);
    }

    # calculate total weight.
    my $TotalWeight = $Label->getCalculatedTotalWeight();
    $Label->set({'TotalWeight' => $TotalWeight}) if ($TotalWeight > 0);

    return $Label;
}

#========================================================================================
# function     isLineItemSupported
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod->isLineItemSupported( $LineItem );
#----------------------------------------------------------------------------------------
# description  Returns true if this lineitem type is supported by this shipping fulfillment method
#               Used for filtering out lineitems that are not meant to be shipped.
#----------------------------------------------------------------------------------------
# input        $LineItem | LineItem | DE_EPAGES::Order::API::Object::LineItem
# return       $IsSupported | True if lineitem can be used. | boolean
#========================================================================================
sub isLineItemSupported {
    my $self = shift;
    my ($LineItem) = @_;
    return ($LineItem->instanceOf('LineItemProduct') && $LineItem->get('IsShippingRequired'))
}


sub _refreshCache{
    my $self = shift;

    my $CacheHandler = DE_EPAGES::Object::API::SavedIncludesHandler->new;
    $CacheHandler->clearObjectCache($self->get('Shop'));
    return;
}

#========================================================================================
# function     getSizeUnit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SizeUnit = $ShippingFulfillmentMethod->getSizeUnit() ;
#----------------------------------------------------------------------------------------
# description  Returns the UnitsOfMeasurement object that represents the units system
#               to use for the shipping size.<br>
#               By default this method returns the 'SizeUnit' attribute defined by the
#               ExternalShipping specific cartridge.<br>
#               See in CARTRIDGE_NAME/Database/XML/ShippingFulfillmentTypes.xml<br>
#               Also a cartridge could implement its own getSizeUnit() method if needs
#               to determine the unit dynamically (!)
#----------------------------------------------------------------------------------------
# return       $SizeUnit | UnitsOfMeasurement object for the size | Object
#========================================================================================
sub getSizeUnit {
    my $self = shift;

    return $self->get('ShippingFulfillmentType')->get( 'SizeUnit' ) ;
}

#========================================================================================
# function     getWeightUnit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WeightUnit = $ShippingFulfillmentMethod->getWeightUnit() ;
#----------------------------------------------------------------------------------------
# description  Returns the UnitsOfMeasurement object that represents the units system
#               to use for the shipping weight.<br>
#               By default this method returns the 'WeightUnit' attribute defined by the
#               ExternalShipping specific cartridge.<br>
#               See in CARTRIDGE_NAME/Database/XML/ShippingFulfillmentTypes.xml<br>
#               Also a cartridge could implement its own getSizeUnit() method if needs
#               to determine the unit dynamically (!)
#----------------------------------------------------------------------------------------
# return       $WeightUnit | UnitsOfMeasurement object for the weight | object
#========================================================================================
sub getWeightUnit {

    my $self = shift;

    return $self->get('ShippingFulfillmentType')->get( 'WeightUnit' ) ;
}


#========================================================================================
# function     addMandatoryAddressFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMandatoryAddressFields( $aShippingAddressFields );
#----------------------------------------------------------------------------------------
# description  Makes certains address fields mandatory. For now this method only supports
#               shipping address fields but it can be extended to support other address
#               fields such as delivery address fields or shop address fields.
#----------------------------------------------------------------------------------------
# input        $aShippingAddressFields | Comma-separated list of shipping
#                                         address fields which must be mandatory. | string
#========================================================================================
sub addMandatoryAddressFields {
    my $self = shift;
    my (@aShippingAddressFields) = @_;

    my $Shop = $self->getSite();
    $self->_addAddressFields($Shop, 'ShippingAddressSetMandatoryFields', @aShippingAddressFields);
    if ($Shop->isUnity()) {
        # for Unity shops shipping address fields are only mandatory if they are
        # mandatory for the billing address and the registration set as well
        $self->_addAddressFields($Shop, 'RegistrationSetMandatoryFields', @aShippingAddressFields);
        $self->_addAddressFields($Shop, 'BillingAddressSetMandatoryFields', @aShippingAddressFields);
    }

    return;
}

#========================================================================================
# function     addVisibleAddressFields
# state        public
#----------------------------------------------------------------------------------------
# syntax       addVisibleAddressFields( $aShippingAddressFields );
#----------------------------------------------------------------------------------------
# description  Makes certains address fields visible. For now this method only supports
#               shipping address fields but it can be extended to support other address
#               fields such as delivery address fields or shop address fields.
#----------------------------------------------------------------------------------------
# input        $aShippingAddressFields | Comma-separated list of shipping
#                                         address fields which must be visible. | string
#========================================================================================
sub addVisibleAddressFields {
    my $self = shift;
    my (@aShippingAddressFields) = @_;

    my $Shop = $self->getSite();
    $self->_addAddressFields($Shop, 'ShippingAddressSetVisibleFields', @aShippingAddressFields);
    if ($Shop->isUnity()) {
        # for Unity shops shipping address fields are only visible if they are
        # visible for the billing address and the registration set as well
        $self->_addAddressFields($Shop, 'RegistrationSetVisibleFields', @aShippingAddressFields);
        $self->_addAddressFields($Shop, 'BillingAddressSetVisibleFields', @aShippingAddressFields);
    }

    return;
}

sub _addAddressFields {
    my $self = shift;
    my ($Shop, $AttributeName, @aAddressFields) = @_;

    my @aCurrentAddressFields = split /,/, $Shop->get($AttributeName);

    my %SeenFields;
    my $NewFields = join(',', grep{ !$SeenFields{$_}++} @aCurrentAddressFields, @aAddressFields);

    $Shop->set({
        $AttributeName => $NewFields,
    });

    return;
}

1;