#========================================================================================
# package      DE_EPAGES::Object::API::Domain
# state        public
# description  Implements concatenation of urls
#========================================================================================
package DE_EPAGES::Object::API::Domain;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    Server
    ServerSSL
    ProtocolAndServer
    ProtocolAndServerSSL
    Urls
    WebServerScriptNameAdmin
    WebServerScriptNameAdminSSL
    WebServerScriptNameStorefront
    WebServerScriptNameStorefrontSSL
    WebServerScriptNameMobileStorefront
    WebServerScriptNameMobileStorefrontSSL
    WebServerScriptNamePreview
    WebServerScriptNamePreviewSSL
    WebServerScriptNameWithOutExtension
    WebServerScriptAdminUrl
    WebServerScriptAdminUrlSSL
    WebServerScriptStorefrontUrl
    WebServerScriptStorefrontUrlSSL
    WebServerScriptMobileStorefrontUrl
    WebServerScriptMobileStorefrontUrlSSL
    WebServerScriptPreviewUrl
    WebServerScriptPreviewUrlSSL
    GetUrlInfo
    PoolDBEntries
    TestWebServerScriptNamePart
);

use DE_EPAGES::Core::API::IDN qw( EncodeDomain );
use DE_EPAGES::Core::API::Log qw( LogDebug );
use DE_EPAGES::Core::API::Net qw( localhostName );
use DE_EPAGES::Core::API::Url qw( UrlEncodeValue );
use DE_EPAGES::Object::API::Constants qw( EXPANDWWWDOMAIN );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject GetRootObjectID );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use DE_EPAGES::Core::API::PerlTools qw( CheckIpFormat );

use constant DEFAULT_HTTP_PORT => 80;
use constant DEFAULT_SSL_PORT => 443;

sub _FormatServerAndPort {
    my( $Server, $Port, $AddPort, $DefaultPort ) = @_;
    $Port //= $DefaultPort; # make sure Port is defined

    my $ErrorMsg = CheckIpFormat($Server, 6);
    my $ServerAndPort = $ErrorMsg ? $Server : "[$Server]";
    $ServerAndPort .= ':'.$Port if $AddPort or $Port != $DefaultPort;
    return $ServerAndPort;
}

#========================================================================================
# function     Server
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Server = Server($hSite, $hSystem, $AddPort);
# example      $Server = Server(GetUrlInfo($Site), 1); # www.epages.de:80 or www.epages.de:8080
# example      $Server = Server(GetUrlInfo($Site), 0); # www.epages.de    or www.epages.de:8080
#----------------------------------------------------------------------------------------
# description  build http server and port
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# input        $AddPort | add the default port number :80 (default 0) | boolean
# return       $Server | server name | string
#========================================================================================
sub Server {
    my ($hSite, $hSystem, $AddPort) = @_;
    $AddPort = 0 unless defined $AddPort;

    # preset for the domain name: system domain or localhost
    my $DomainName = defined $hSystem->{'DomainName'} ? $hSystem->{'DomainName'} : localhostName();

    # do not use own domain in case of a Unity shop if http request
    if ($hSite->{'IsUnity'}) {
        # add subdomain if setting enabled
        $DomainName = $hSystem->{'UnityUrlScheme'} if $hSystem->{'UnityUrlScheme'};
        $DomainName = _constructUnityDomainName($DomainName, $hSite, $hSystem);
    } else {
        # use own domain if defined
        $DomainName = $hSite->{'DomainName'} if defined $hSite->{'DomainName'};
    }
    my $Server = EncodeDomain($DomainName);

    return _FormatServerAndPort($Server, $hSystem->{'WebServerPort'}, $AddPort, DEFAULT_HTTP_PORT);
}

#========================================================================================
# function     _Servers
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aServers = _Servers($hSite, $hSystem, $AddPort);
# example      $aServers = _Servers(GetUrlInfo($Site), 1); # www.epages.de:80 or www.epages.de:8080
# example      $aServers = _Servers(GetUrlInfo($Site), 0); # www.epages.de    or www.epages.de:8080
#----------------------------------------------------------------------------------------
# description  build http servers and port
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# input        $AddPort | add the default port number :80 (default 0) | boolean
# return       $aServers | server names | ref.array.string
#========================================================================================
sub _Servers {
    my ($hSite, $hSystem, $AddPort) = @_;
    $AddPort //= 0;

    my @aServers;
    my $Server = Server($hSite, $hSystem, $AddPort); # primary address
    push(@aServers, $Server);

    my @aSecondaryDomains;
    push(@aSecondaryDomains, @{$hSite->{'SecondaryDomains'}}) if defined $hSite->{'SecondaryDomains'};
    push(@aSecondaryDomains, @{$hSystem->{'SecondaryDomains'}}) if defined $hSystem->{'SecondaryDomains'};

    foreach my $SecondaryDomain (@aSecondaryDomains) {
        push(@aServers, _FormatServerAndPort(EncodeDomain($SecondaryDomain), $hSystem->{'WebServerPort'},
            $AddPort, DEFAULT_HTTP_PORT));
    }

    return \@aServers;
}

#========================================================================================
# function     ProtocolAndServer
# state        public
# syntax       $Server = ProtocolAndServer(($hSite, $hSystem));
# example      $Server = ProtocolAndServer(GetUrlInfo($Site));  # http://www.epages.de:8080
#----------------------------------------------------------------------------------------
# description  build protocol, http server and port
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ProtocolAndServer | protocol, http server and port | string
#========================================================================================
sub ProtocolAndServer {
    my ($hSite, $hSystem) = @_;

    return 'http://'. Server($hSite, $hSystem);
}

#========================================================================================
# function     ServerSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Server = ServerSSL($hSite, $hSystem, $AddPort);
# example      $Server = ServerSSL(GetUrlInfo($Site), 1);  # www.epages.de:443 or www.epages.de:4444
# example      $Server = ServerSSL(GetUrlInfo($Site), 0);  # www.epages.de     or www.epages.de:4444
#----------------------------------------------------------------------------------------
# description  returns the SSL server name.
#               Returns undef if SSL cannot be used.
#               If the site does not have an SSL certificate and the site has a domain
#               and $ENV{'EPAGES_SSL_PROXY'} is set, returns $ENV{'EPAGES_SSL_PROXY'}
#               (which is not necessary a valid server name).
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# input        $AddPort | add the default port number :443 (default 0) | boolean
# return       $Server | SSL server name | string
#========================================================================================
sub ServerSSL {
    my ($hSite, $hSystem, $AddPort) = @_;
    $AddPort = 0 unless defined $AddPort;

    my $FormatDomain = sub {
        my ($Domain) = @_;
        my $Server = EncodeDomain($Domain);
        return _FormatServerAndPort($Server, $hSystem->{'SSLWebServerPort'}, $AddPort, DEFAULT_SSL_PORT);
    };

    my $DomainName;

    if ($hSite->{'HasSSLCertificate'} && $hSite->{'DomainName'}) {
        # own domain with SSL -> we take it
        $DomainName = $FormatDomain->($hSite->{'DomainName'});
    } elsif ($hSystem->{'HasSSLCertificate'}) {
        # own handling for unity shops -> no SSL Proxy
        if ($hSite->{'IsUnity'}) {
            $DomainName = $hSystem->{'DomainName'} // localhostName();
            $DomainName = $hSystem->{'UnityUrlScheme'} if $hSystem->{'UnityUrlScheme'};
            # determine provider domain
            $DomainName = $FormatDomain->($DomainName);
            # add subdomain if setting enabled
            $DomainName = _constructUnityDomainName($DomainName, $hSite, $hSystem);
        } else {
            # use SSL proxy if defined
            $DomainName = $ENV{'EPAGES_SSL_PROXY'}
                ? $ENV{'EPAGES_SSL_PROXY'}
                # use system domain or hostname
                : $FormatDomain->($hSystem->{'DomainName'} // localhostName());
        }
    } else {
        # no SSL certificate on site and system -> SSL not possible
        $DomainName = undef;
    }

    return $DomainName;
}

sub _constructUnityDomainName {
    my ($DomainName, $hSite, $hSystem) = @_;

    # set the Unity Domain Name to <WebServerScriptNamePart>.<ProviderDomain> in case
    # the setting 'SubdomainRouting' is set in the TBO
    my $SubDomain = lc $hSite->{'WebServerScriptNamePart'};
    if ($SubDomain and $hSystem->{'UnityUseSubdomainRouting'}) {
        # convert the WebServerScriptNamePart to a valid subdomain (lower case, - instead of _)
        $SubDomain =~ s/_/-/g;
        $DomainName = $SubDomain . '.' . $DomainName;
    }

    return $DomainName;
}

#========================================================================================
# function     _ServersSSL
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aServers = _ServersSSL($hSite, $hSystem, $AddPort);
# example      $aServers = _ServersSSL(GetUrlInfo($Site), 1);  # www.epages.de:443 or www.epages.de:4444
# example      $aServers = _ServersSSL(GetUrlInfo($Site), 0);  # www.epages.de
#----------------------------------------------------------------------------------------
# description  returns the SSL server names.
#               Returns an empty array if SSL cannot be used.
#               If the site does not have an SSL certificate and the site has a domain
#               and $ENV{'EPAGES_SSL_PROXY'} is set, the first array entry will be
#               $ENV{'EPAGES_SSL_PROXY'} (which is not necessarily a valid server name).
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# input        $AddPort | add the default port number :443 (default 0) | boolean
# return       $aServers | SSL server names | ref.array.string
#========================================================================================
sub _ServersSSL {
    my ($hSite, $hSystem, $AddPort) = @_;
    $AddPort //= 0;

    my $FormatDomain = sub {
        my ($Domain) = @_;
        my $Server = EncodeDomain($Domain);
        return _FormatServerAndPort($Server, $hSystem->{'SSLWebServerPort'}, $AddPort, DEFAULT_SSL_PORT);
    };

    my @aServers;
    my $ServerSSL = ServerSSL($hSite, $hSystem, $AddPort);

    return \@aServers unless defined $ServerSSL;

    push(@aServers, $ServerSSL);

    my @aSecondaryDomains;
    push(@aSecondaryDomains, @{$hSite->{'SecondaryDomains'}}) if defined $hSite->{'SecondaryDomains'};
    push(@aSecondaryDomains, @{$hSystem->{'SecondaryDomains'}}) if defined $hSystem->{'SecondaryDomains'};

    push(@aServers, $FormatDomain->($_)) foreach @aSecondaryDomains;

    return \@aServers;
}

#========================================================================================
# function     ProtocolAndServerSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProtocolAndServerSSL = ProtocolAndServerSSL($hSite, $hSystem);
# example      $Server = ProtocolAndServerSSL(GetUrlInfo($Site));  # https://www.epages.de
#----------------------------------------------------------------------------------------
# description  build protocol, https server and port.
#               If the site does not have an SSL certificate and $ENV{'EPAGES_SSL_PROXY'}
#               is set, returns a special SSL proxy URL in the format
#               "http://sslproxy/domainname".
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ProtocolAndServerSSL | SSL server name and protocol, e.g.
#                   "https://www.epages.de" or "https://www.epages.de:4443" | string
#========================================================================================
sub ProtocolAndServerSSL {
    my ($hSite, $hSystem) = @_;

    my $SSLServer = ServerSSL( $hSite, $hSystem );

    return defined $SSLServer
        ? $ENV{'EPAGES_SSL_PROXY'} && $SSLServer eq $ENV{'EPAGES_SSL_PROXY'}
            ? "https://$ENV{'EPAGES_SSL_PROXY'}/".UrlEncodeValue(Server($hSite, $hSystem))
            : "https://$SSLServer"
        : ProtocolAndServer($hSite, $hSystem);
}

#========================================================================================
# function     WebServerScriptNameWithOutExtension
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptName = WebServerScriptNameWithOutExtension($hSite, $hSystem);
# example      $ScriptName = WebServerScriptNameWithOutExtension(GetUrlInfo($Site));  # /epages/Store.
#----------------------------------------------------------------------------------------
# description  build url without hostname and extension
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptName | script name part of the URL without the extension,
#                   e.g. "/epages/DemoShop." | string
#========================================================================================
sub WebServerScriptNameWithOutExtension {
    my ($hSite, $hSystem) = @_;

    my $WebServerScriptNamePart = $hSite->{'WebServerScriptNamePart'};
    $WebServerScriptNamePart = $hSystem->{'WebServerScriptNamePart'} unless defined $WebServerScriptNamePart;

    if( defined $hSystem->{'WebServerDirectory'} ) {
        return '/' . $hSystem->{'WebServerDirectory'} .'/'. $WebServerScriptNamePart . '.';
    }
    else {
        return '/'. $WebServerScriptNamePart . '.';
    }
}

#========================================================================================
# function     WebServerScriptNameWithOutExtensionSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameWithOutExtensionSSL($hSite, $hSystem);
# example      $Server = WebServerScriptNameWithOutExtension(GetUrlInfo($Site));  # /epages/Store.
#----------------------------------------------------------------------------------------
# description  build url without hostname and extension, for ssl connection
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptName | script name part of the URL without the extension,
#                   e.g. "/epages/DemoShop." | string
#========================================================================================
sub WebServerScriptNameWithOutExtensionSSL{
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtension($hSite, $hSystem);
}

#========================================================================================
# function     WebServerScriptNameAdmin
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameAdmin($hSite, $hSystem);
# example      $Server = WebServerScriptNameAdmin(GetUrlInfo($Site));  # /epages/Store.admin
#----------------------------------------------------------------------------------------
# description  build url without hostname
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the back-office URL,
#                   e.g. "/epages/DemoShop.admin" | string
#========================================================================================
sub WebServerScriptNameAdmin {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtension($hSite, $hSystem) . $hSystem->{'URLAdminExtension'};
}

#========================================================================================
# function     WebServerScriptNameAdminSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameAdminSSL($hSite, $hSystem);
# example      $Server = WebServerScriptNameAdminSSL(GetUrlInfo($Site));  # /epages/Store.admin
#----------------------------------------------------------------------------------------
# description  build url without hostname, ssl connection preferred.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the back-office URL,
#                   e.g. "/epages/DemoShop.admin" | string
#========================================================================================
sub WebServerScriptNameAdminSSL {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtensionSSL($hSite, $hSystem) . $hSystem->{'URLAdminExtension'};
}

#========================================================================================
# function     WebServerScriptNameStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameStorefront($hSite, $hSystem);
# example      $Server = WebServerScriptNameStorefront(GetUrlInfo($Site));  # /epages/Store.sf
#----------------------------------------------------------------------------------------
# description  build url without hostname.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the storefront URL,
#                   e.g. "/epages/DemoShop.sf" | string
#========================================================================================
sub WebServerScriptNameStorefront {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtension($hSite, $hSystem) . $hSystem->{'URLStorefrontExtension'};
}

#========================================================================================
# function     WebServerScriptNameStorefrontSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameStorefrontSSL($hSite, $hSystem);
# example      $Server = WebServerScriptNameStorefrontSSL(GetUrlInfo($Site));  # /epages/Store.sf
#----------------------------------------------------------------------------------------
# description  build url without hostname, ssl connection preferred.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the storefront URL in SSL mode,
#                   e.g. "/epages/DemoShop.sf" | string
#========================================================================================
sub WebServerScriptNameStorefrontSSL {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtensionSSL($hSite, $hSystem) . $hSystem->{'URLStorefrontExtension'};
}

#========================================================================================
# function     WebServerScriptNameMobileStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameMobileStorefront($hSite, $hSystem);
# example      $Server = WebServerScriptNameMobileStorefront(GetUrlInfo($Site));  # /epages/Store.mobile
#----------------------------------------------------------------------------------------
# description  build url without hostname.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the mobile storefront URL,
#                   e.g. "/epages/DemoShop.mobile" | string
#========================================================================================
sub WebServerScriptNameMobileStorefront {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtension($hSite, $hSystem) . $hSystem->{'URLMobileStorefrontExtension'};
}

#========================================================================================
# function     WebServerScriptNameMobileStorefrontSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNameMobileStorefrontSSL($hSite, $hSystem);
# example      $Server = WebServerScriptNameMobileStorefrontSSL(GetUrlInfo($Site));  # /epages/Store.mobile
#----------------------------------------------------------------------------------------
# description  build url without hostname, ssl connection preferred.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the mobile storefront URL in SSL mode,
#                   e.g. "/epages/DemoShop.mobile" | string
#========================================================================================
sub WebServerScriptNameMobileStorefrontSSL {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtensionSSL($hSite, $hSystem) . $hSystem->{'URLMobileStorefrontExtension'};
}

#========================================================================================
# function     WebServerScriptNamePreview
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNamePreview($hSite, $hSystem);
# example      $Server = WebServerScriptNamePreview(GetUrlInfo($Site));  # /epages/Store.preview
#----------------------------------------------------------------------------------------
# description  build url without hostname.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the preview URL,
#                   e.g. "/epages/DemoShop.preview" | string
#========================================================================================
sub WebServerScriptNamePreview {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtension($hSite, $hSystem) . $hSystem->{'URLPreviewExtension'};
}

#========================================================================================
# function     WebServerScriptNamePreviewSSL
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptNamePart = WebServerScriptNamePreviewSSL($hSite, $hSystem);
# example      $Server = WebServerScriptNamePreviewSSL(GetUrlInfo($Site));  # /epages/Store.preview
#----------------------------------------------------------------------------------------
# description  build url without hostname, ssl connection preferred.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $ScriptNamePart | script name part of the preview URL in SSL mode,
#                   e.g. "/epages/DemoShop.preview" | string
#========================================================================================
sub WebServerScriptNamePreviewSSL {
    my ($hSite, $hSystem) = @_;

    return WebServerScriptNameWithOutExtensionSSL($hSite, $hSystem) . $hSystem->{'URLPreviewExtension'};
}

## no critic qw(RequireCommentForExportedFunctions)
sub WebServerScriptAdminUrl {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServer($hSite, $hSystem) . WebServerScriptNameAdmin($hSite, $hSystem);
}

sub WebServerScriptAdminUrlSSL {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServerSSL($hSite, $hSystem) . WebServerScriptNameAdminSSL($hSite, $hSystem);
}

sub WebServerScriptStorefrontUrl {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServer($hSite, $hSystem) . WebServerScriptNameStorefront($hSite, $hSystem);
}

sub WebServerScriptMobileStorefrontUrl {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServer($hSite, $hSystem) . WebServerScriptNameMobileStorefront($hSite, $hSystem);
}

sub WebServerScriptMobileStorefrontUrlSSL {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServerSSL($hSite, $hSystem) . WebServerScriptNameMobileStorefrontSSL($hSite, $hSystem);
}

sub WebServerScriptStorefrontUrlSSL {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServerSSL($hSite, $hSystem) . WebServerScriptNameStorefrontSSL($hSite, $hSystem);
}

sub WebServerScriptPreviewUrl {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServer($hSite, $hSystem) . WebServerScriptNamePreview($hSite, $hSystem);
}

sub WebServerScriptPreviewUrlSSL {
    my ($hSite, $hSystem) = @_;

    return ProtocolAndServerSSL($hSite, $hSystem) . WebServerScriptNamePreviewSSL($hSite, $hSystem);
}
## use critic

#========================================================================================
# function     PoolDBEntries
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aEntries = PoolDBEntries($hSite, $hSystem);
# example      $aEntries = PoolDBEntries(GetUrlInfo($Site));
#----------------------------------------------------------------------------------------
# description  collects pooldb entries for given site.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $aEntries | keys :
#               <ul>
#                 <li>Host - host name and port - string
#                 <li>URI - url with out host name and extension - string
#               </ul> | ref.array.hash
#----------------------------------------------------------------------------------------
# hook         PoolDBEntries | hook parameter keys :
#               <ul>
#                 <li>hEntries - pre collected entries (first key host, second uri) - ref.hash.hash
#                 <li>hSite    - site info - ref.hash
#                 <li>hSystem  - system info - ref.hash
#               </ul>
#========================================================================================
sub PoolDBEntries {
    my ($hSite, $hSystem) = @_;

    # use hash to avoid double entries
    # structure: $Entries{$Domain}{$URI} = undef
    my %Entries;
    my %SystemEntries;

    # get system entries
    my $SystemURI = WebServerScriptNameWithOutExtension($hSystem, $hSystem);
    my $SystemSSLURI = WebServerScriptNameWithOutExtensionSSL($hSystem, $hSystem);
    my $SystemServer = Server($hSystem, $hSystem, 0);
    my $SystemServerSSL = ServerSSL($hSystem, $hSystem, 0);

    $SystemEntries{$SystemServer}{$SystemURI} = undef;

    # SSL system entries
    if ($hSystem->{'HasSSLCertificate'} || $hSite->{'HasSSLCertificate'}) {
        $SystemEntries{$SystemServerSSL}{$SystemSSLURI} = undef;
    }

    if ($hSite->{'SiteID'} == GetRootObjectID()) {
        %Entries = %SystemEntries;
        my $aServers = _Servers($hSite, $hSystem, 0);
        foreach my $Server (@$aServers) {
            _addEntry( $Server, $SystemURI, \%Entries, \%SystemEntries );
        }

        if ($hSystem->{'HasSSLCertificate'}) {
            my $aServersSSL = _ServersSSL($hSite, $hSystem, 0);
            foreach my $ServerSSL (@$aServersSSL) {
                _addEntry( $ServerSSL, $SystemSSLURI, \%Entries, \%SystemEntries );
            }
        }
    }
    else {
        my $URI = WebServerScriptNameWithOutExtension($hSite, $hSystem);
        my $SSLURI = WebServerScriptNameWithOutExtensionSSL($hSite, $hSystem);
        my $aServers = _Servers($hSite, $hSystem, 0);

        foreach my $Server (@$aServers) {
            _addEntry( $Server, $URI, \%Entries, \%SystemEntries );
        }

        # add ssl
        if ($hSystem->{'HasSSLCertificate'} || $hSite->{'HasSSLCertificate'}) {
            my $aServersSSL = _ServersSSL($hSite, $hSystem, 0);
            foreach my $ServerSSL (@$aServersSSL) {
                _addEntry( $ServerSSL, $SSLURI, \%Entries, \%SystemEntries );
            }
        }

        if( defined $hSite->{'WebServerScriptNamePart'} ) {
            ##http://domainname/epages/<ShortURL>.sf
            _addEntry( $SystemServer, $URI, \%Entries, \%SystemEntries );

            if ($hSystem->{'HasSSLCertificate'} || $hSite->{'HasSSLCertificate'}) {
                ##https://domainname/epages/<ShortURL>.sf
                _addEntry( $SystemServerSSL, $SSLURI, \%Entries, \%SystemEntries );
            }
        }
    }

    TriggerHook('PoolDBEntries', {'hEntries'=>\%Entries, 'hSite'=>$hSite, 'hSystem'=>$hSystem});

    # fill to list
    my @Entries;
    foreach my $Host (sort keys %Entries) {
        foreach my $URI (sort keys %{$Entries{$Host}}) {
            push @Entries, {'Host'=>$Host, 'URI'=>$URI};
        }
    }

    LogDebug( 'PoolDBEntries', [ [$hSite, $hSystem], \@Entries ] );

    return \@Entries;
}

sub _addEntry($$$$) {
    my( $Server, $URI, $hEntries, $hSystemEntries ) = @_;

    if( !exists $hSystemEntries->{$Server} || !exists $hSystemEntries->{$Server}->{$URI} ) {
        $hEntries->{$Server}->{$URI} = undef;
    }

    if( EXPANDWWWDOMAIN ) {
        # add or remove 'www.' at the beginning
        my $AltServer = $Server =~ /^www\.(.+)/i ? $1 : 'www.'.$Server;
        if( !exists $hSystemEntries->{$AltServer} && !exists $hSystemEntries->{$Server} ) {
            $hEntries->{$AltServer}->{$URI} = undef;
        }
    }

    return;
}

#========================================================================================
# function     Urls
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hUrls = Urls($hSite, $hSystem);
# example      $hUrls = Urls(GetUrlInfo($Site));
#----------------------------------------------------------------------------------------
# description  return the effective URLs of this site by merging system and site
#               settings.
#----------------------------------------------------------------------------------------
# input        $hSite | site info | ref.hash
# input        $hSystem | system info | ref.hash
# return       $hUrls | system parameter keys :
#               <ul>
#                 <li>Server - HTTP host (web server name and port, if non-standard),
#                               example: 'www.myshop.de' or 'www.myshop.de:81' - string</li>
#                 <li>ServerSSL - SSL web server name and port if non-standard,
#                               example: 'www.myshop.de' or 'www.myshop.de:444' - string</li>
#                 <li>ProtocolAndServer - protocol and web server name,
#                               example: 'http://www.myshop.de:81' - string</li>
#                 <li>ProtocolAndServerSSL - SSL protocol and web server name,
#                               example: 'https://www.myshop.de:444' - string</li>
#                 <li>WebServerScriptNameWithOutExtension - web server script name without
#                               extension, but including the dot,
#                               example: '/epages/myshop.' - string</li>
#                 <li>WebServerScriptNameStorefront - web server script name (storefront),
#                               example: '/epages/myshop.sf' - string</li>
#                 <li>WebServerScriptNameStorefrontSSL - web server script name (admin, SSL),
#                               example: '/epages/Store.sf' - string</li>
#                 <li>WebServerScriptNameAdmin - web server script name (storefront),
#                               example: '/epages/myshop.admin' - string</li>
#                 <li>WebServerScriptNameAdminSSL - web server script name (admin, SSL),
#                               example: '/epages/Store.admin' - string</li>
#                 <li>WebServerScriptStorefrontUrl - protocol, web server name and script name (storefront),
#                               example: 'http://www.myshop.de:81/epages/myshop.sf' - string</li>
#                 <li>WebServerScriptStorefrontUrlSSL - web server script name (storefront, SSL),
#                               example: 'https://www.provider.de/epages/Store.sf' - string</li>
#                 <li>WebServerScriptAdminUrl - protocol, web server name and script name (admin),
#                               example: 'http://www.myshop.de:81/epages/myshop.admin' - string</li>
#                 <li>WebServerScriptAdminSSLUrl - protocol, web server name and script name (admin, SSL),
#                               example: 'http://www.myshop.de:81/epages/myshop.admin' - string</li>
#                 <li>WebServerScriptPreviewUrl - protocol, web server name and script name (preview),
#                               example: 'http://www.myshop.de:81/epages/myshop.preview' - string</li>
#               </ul> | ref.hash
#========================================================================================
sub Urls ($$) {
    my ($hSite, $hSystem) = @_;
    my %Info = (
        'Server'    => Server($hSite, $hSystem),
        'ServerSSL' => ServerSSL($hSite, $hSystem),
        'ProtocolAndServer'    => ProtocolAndServer($hSite, $hSystem),
        'ProtocolAndServerSSL' => ProtocolAndServerSSL($hSite, $hSystem),
        'WebServerScriptNameWithOutExtension'    => WebServerScriptNameWithOutExtension($hSite, $hSystem),
        'WebServerScriptNameStorefront'    => WebServerScriptNameStorefront($hSite, $hSystem),
        'WebServerScriptNameStorefrontSSL' => WebServerScriptNameStorefrontSSL($hSite, $hSystem),
        'WebServerScriptNameAdmin'    => WebServerScriptNameAdmin($hSite, $hSystem),
        'WebServerScriptNameAdminSSL' => WebServerScriptNameAdminSSL($hSite, $hSystem),
        'WebServerScriptStorefrontUrl' => WebServerScriptStorefrontUrl($hSite, $hSystem),
        'WebServerScriptStorefrontUrlSSL' => WebServerScriptStorefrontUrlSSL($hSite, $hSystem),
        'WebServerScriptAdminUrl' => WebServerScriptAdminUrl($hSite, $hSystem),
        'WebServerScriptAdminSSLUrl' => WebServerScriptAdminUrlSSL($hSite, $hSystem),
        'WebServerScriptPreviewUrl' => WebServerScriptPreviewUrl($hSite, $hSystem),
    );
    return \%Info;
}

#========================================================================================
# function     GetUrlInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($hSite, $hSystem) = GetUrlInfo($Site);
#----------------------------------------------------------------------------------------
# description  return all informations to build urls of this site. See <function Urls> and
#               <function PoolDBEntries>
#----------------------------------------------------------------------------------------
# input        $Site | site | object
# return       $hSite | site parameter keys :
#               <ul>
#                 <li>DomainName    - domain name - string
#                 <li>HasSSLCertificate - can use ssl - boolean
#                 <li>WebServerScriptNamePart - script name part - string
#                 <li>SiteID - identifier - integer
#                 <li>SecondaryDomains - additional domain names - ref.array.string
#                 <li>IsUnity - is this a Unity shop (undef: not shop, 
#                       0: no, 1: yes) - boolean
#               </ul> | ref.hash
# return       $hSystem | system parameter keys :
#               <ul>
#                 <li>DomainName    - domain name - string
#                 <li>HasSSLCertificate - can use ssl - boolean
#                 <li>WebServerScriptNamePart - script name part - string
#                 <li>WebServerDirectory - web server directory (epages)- string
#                 <li>URLStorefrontExtension - url extension (sf)- string
#                 <li>URLMobileStorefrontExtension - url extension (mobile)- string
#                 <li>URLAdminExtension - url extension (admin)- string
#                 <li>WebServerPort - http port - string
#                 <li>SSLWebServerPort - https port - string
#                 <li>SiteID - identifier - integer
#                 <li>SecondaryDomains - additional domain names - ref.array.string
#                 <li>UnityUseSubdomainRouting - subdomain setting set for Unity
#                       databases (undef: not existing, 0: no, 1: yes)
#                 <li>UnityUrlScheme - URL scheme for Unity - string
#               </ul> | ref.hash
#========================================================================================
sub GetUrlInfo ($) {
    my ($Site) = @_;
    my $hSite = $Site->get([qw(SiteID DomainName HasSSLCertificate WebServerScriptNamePart SecondaryDomains)]);

    $hSite->{'IsUnity'} = $Site->class->alias eq 'Shop'
        ? $Site->featureMaxValue('Unity') : undef;

    my $System = LoadRootObject();
    my $hSystem = $System->get([qw(SiteID DomainName HasSSLCertificate
        WebServerScriptNamePart WebServerDirectory WebServerPort SSLWebServerPort
        URLAdminExtension URLStorefrontExtension URLMobileStorefrontExtension URLPreviewExtension SecondaryDomains)]);

    $hSystem->{'UnityUseSubdomainRouting'} = $System->class->existsAttribute('UnityUseSubdomainRouting')
        ? $System->get('UnityUseSubdomainRouting') : undef;
    $hSystem->{'UnityUrlScheme'} = $System->class->existsAttribute('UnityUrlScheme')
        ? $System->get('UnityUrlScheme') : undef;

    return ($hSite, $hSystem);
}

#========================================================================================
# function     TestWebServerScriptNamePart
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ( TestWebServerScriptNamePart($part) ) { ... }
#----------------------------------------------------------------------------------------
# description  returns true if web server script name part does not contain illegal
#               characters
#----------------------------------------------------------------------------------------
# input        $WebServerScriptNamePart | part of webserver script name | string
# return       $test | true if part OK | boolean
#========================================================================================
sub TestWebServerScriptNamePart {
    my ($WebServerScriptNamePart) = @_;
    return $WebServerScriptNamePart =~ /^[0-9a-zA-Z:@&=+\$,_-]+$/x;
}

1;
