#========================================================================================
# package      DE_EPAGES::Order::Hooks::Style
# state        public
# description  Additional hook function for AfterUpdateCustomizableButtons hook.
#========================================================================================
package DE_EPAGES::Order::Hooks::Style;

use strict;
use DE_EPAGES::Presentation::API::Constants qw ( UNITY_SF_VIEW_ACTION_PREFIX ); ##no critic(RequireCorrectImports)

#========================================================================================
# function     OnBeforeUpdateStyle
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateStyle( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by hook OnBeforeImportObject.
#               Sets the customizable basket button settings to the customizable basket
#               settings. This are the default basket button values.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - style object</li>
#                   <li>Values - attributes</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateStyle {
    my ($hParams) = @_;

    my $Style = $hParams->{'Object'};
    my $hAttributes = $hParams->{'Values'};

    my $hCustomizableButtons = $hAttributes->{'CustomizableButtons'};

    # save default basket button style values
    if (exists $hAttributes->{'CustomizableBasketButton'} and not defined $Style->get('DefaultBasketButtonStyle')) {
        $hAttributes->{'DefaultBasketButtonStyle'} = $hAttributes->{'CustomizableBasketButton'};
    }
    if(not defined $Style->get('CustomizableBasketButton') and not defined $Style->get('DefaultBasketButtonStyle')
    and not exists $hAttributes->{'CustomizableBasketButton'} and defined $hCustomizableButtons) {
        $hAttributes->{'CustomizableBasketButton'} = { %$hCustomizableButtons,
            'Image' => {
                'Path' => undef,
                'Height' => undef,
                'Width' => undef,
            },
            'IsCustomized' => 0
        };
        $hAttributes->{'DefaultBasketButtonStyle'} = $hAttributes->{'CustomizableBasketButton'};
    }
}

#========================================================================================
# function     OnWriteCSS
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnWriteCSS($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook API_OnWriteCSS.
#               Here, we're writing the OrderProcessStyle.css file.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - style object</li>
#                   <li>Vars - TLE vars hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnWriteCSS {
    my ($hParams) = @_;
    my $Style = $hParams->{'Object'};
    my $Vars = $hParams->{'Vars'};

    $Style->writeCSSFile('OrderProcessStyle', 'SF-OrderProcessStyle', $Vars);
}

#========================================================================================
# function     OnRemoveObsoleteCSS
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnRemoveObsoleteCSS($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook API_OnRemoveObsoleteCSS.
#               Here, we're removing all obsolete OrderProcessStyle.css files.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - style object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnRemoveObsoleteCSS {
    my ($hParams) = @_;
    my $Style = $hParams->{'Object'};

    $Style->removeCSSFiles(['OrderProcessStyle']);
}

#========================================================================================
# function     OnGetStyleURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetStyleURL($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook API_OnGetStyleURL.
#               Creates the possibility to alter the SF style URL. Here, we're it replacing
#               with the OrderProcessURL if we're in the multi-step checkout.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - style object</li>
#                   <li>Values - attributes</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetStyleURL {
    my ($hParams) = @_;

    my $Style = $hParams->{'Object'};
    my $hBasketOrderProcess = $hParams->{'Values'}->{'Input'}->{'BasketOrderProcess'};

    if (defined $hBasketOrderProcess
        and (exists $hBasketOrderProcess->{'CurrentStep'})
        and $hBasketOrderProcess->{'CurrentStep'}->{'ViewAction'}->{'Alias'} ne 'ViewMultiCheckoutBasket')
    {
        $hParams->{'Values'}->{'StyleURL'} = $Style->get('OrderProcessURL');
    } elsif ($Style->getSite()->get('IsUnityRegistered')
        && $hParams->{'Values'}->{'Input'}->{'ViewAction'} =~ m/^${\UNITY_SF_VIEW_ACTION_PREFIX}/) {

        $hParams->{'Values'}->{'StyleURL'} = $Style->get('OrderProcessURL');
    }
}

1;
