#========================================================================================
# package      DE_EPAGES::ProductPortal::t::unit::Portal
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for API::Portal
#========================================================================================
package DE_EPAGES::ProductPortal::t::unit::Portal;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime SubtractDuration);
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Mockify::Sut;
use Test::MockObject;
use Test::More;
use Test::Exception;
use DE_EPAGES::ProductPortal::API::Portal qw ( LoadPortalSite GetPortalConfigFolder );

use strict;

use constant PORTAL_SITE_ID => '1222';
use constant PORTAL_ALIAS => 'Portal1';
use constant PORTAL_LOCALEID => 'de_DE';

sub test {
    my $self = shift;

    $self->test_LoadPortalSite_success();
    $self->test_LoadPortalSite_fails_instancePortalConfig();

    $self->test_GetPortalConfigFolder_success();
    $self->test_GetPortalConfigFolder_fails_duplicated();
    $self->test_GetPortalConfigFolder_fails_deprecated();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_LoadPortalSite_success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hParams = {
        'IsPortalSiteDeprecated' => 0,
        'IsDuplicatedPortalFolder' => 0,
        'PortalSiteInstanceOf' => 'PortalSite'
    };

    my $UniversalPortalSite = $self->_createUniversalPortalSite($hParams);
    $hParams->{'UniversalPortalSite'} = $UniversalPortalSite;

    my $Shop = $self->_createShop($hParams);

    lives_ok(sub { LoadPortalSite($Shop, PORTAL_SITE_ID) } , $SubTestName.' - no error with LoadPortalSite');
    my $GotLoadPortalSite = LoadPortalSite($Shop, PORTAL_SITE_ID);

    is_deeply($GotLoadPortalSite, $UniversalPortalSite, "$SubTestName - returned UniversalPortalSite has expected");
}

#----------------------------------------------------------------------------------------
sub test_LoadPortalSite_fails_instancePortalConfig {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hParams = {
        'IsPortalSiteDeprecated' => 0,
        'IsDuplicatedPortalFolder' => 0,
        'PortalSiteInstanceOf' => 'PortalConfig'
    };

    my $UniversalPortalSite = $self->_createUniversalPortalSite($hParams);
    $hParams->{'UniversalPortalSite'} = $UniversalPortalSite;

    my $Shop = $self->_createShop($hParams);

    lives_ok(sub { LoadPortalSite($Shop, PORTAL_SITE_ID) } , $SubTestName.' - no error with LoadPortalSite');
    my $GotLoadPortalSite = LoadPortalSite($Shop, PORTAL_SITE_ID);

    is($GotLoadPortalSite, undef, "$SubTestName - returned undef with PortalSite instanceOf PortalConfig");
}

#----------------------------------------------------------------------------------------
sub test_GetPortalConfigFolder_success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hParams = {
        'IsPortalSiteDeprecated' => 0,
        'IsDuplicatedPortalFolder' => 0
    };

    my $UniversalPortalSite = $self->_createUniversalPortalSite($hParams);
    my $PortalConfigFolder = $self->_createPortalConfigFolder($hParams);

    $hParams->{'PortalConfigFolder'} = $PortalConfigFolder;
    my $Shop = $self->_createShop($hParams);

    my $hInfo = { 'Position' => 180 };

    lives_ok(sub { GetPortalConfigFolder($Shop, $UniversalPortalSite, $hInfo) } , $SubTestName.' - no error with GetPortalConfigFolder');
    my $GotPortalFolder = GetPortalConfigFolder($Shop, $UniversalPortalSite, $hInfo);

    is_deeply($GotPortalFolder, $PortalConfigFolder, "$SubTestName - returned PortalConfigFolder has expected");
}

#----------------------------------------------------------------------------------------
sub test_GetPortalConfigFolder_fails_duplicated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hParams = {
        'IsPortalSiteDeprecated' => 0,
        'IsDuplicatedPortalFolder' => 1
    };

    my $UniversalPortalSite = $self->_createUniversalPortalSite($hParams);
    my $PortalConfigFolder = $self->_createPortalConfigFolder($hParams);

    $hParams->{'PortalConfigFolder'} = $PortalConfigFolder;
    my $Shop = $self->_createShop($hParams);

    my $hInfo = { 'Position' => 180 };

    lives_ok(sub { GetPortalConfigFolder($Shop, $UniversalPortalSite, $hInfo) } , $SubTestName.' - no error with GetPortalConfigFolder');
    my $GotPortalFolder = GetPortalConfigFolder($Shop, $UniversalPortalSite, $hInfo);

    is($GotPortalFolder, undef, "$SubTestName - Duplicated PortalSite Alias returns Undef");
}
#----------------------------------------------------------------------------------------
sub test_GetPortalConfigFolder_fails_deprecated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hParams = {
        'IsPortalSiteDeprecated' => 1,
        'IsDuplicatedPortalFolder' => 0
    };

    my $UniversalPortalSite = $self->_createUniversalPortalSite($hParams);
    my $PortalConfigFolder = $self->_createPortalConfigFolder($hParams);

    $hParams->{'PortalConfigFolder'} = $PortalConfigFolder;
    my $Shop = $self->_createShop($hParams);

    my $hInfo = { 'Position' => 180 };

    lives_ok(sub { GetPortalConfigFolder($Shop, $UniversalPortalSite, $hInfo) } , $SubTestName.' - no error with GetPortalConfigFolder');
    my $GotPortalFolder = GetPortalConfigFolder($Shop, $UniversalPortalSite, $hInfo);

    is($GotPortalFolder, undef, "$SubTestName - Deprecated PortalSite returns Undef");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($hParams) = @_;


    my $hAttributes = {
        'Alias' => 'MyShop',
        'IsUnityRegistered' => $hParams->{'IsUnityRegistered'},
        'PortalConfigs' => undef
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop',$hAttributes);

    $Mockify->addMockWithReturnValueAndParameterCheck(
        'loadObject', 
        $hParams->{'UniversalPortalSite'},
        [
            {'int' => PORTAL_SITE_ID},
            {'string' => 'Object'},
            {'hashref' => { AllowSystemSite => 1 }}
        ]
    );

    $Mockify->addMockWithReturnValueAndParameterCheck(
        'folder',
        $hParams->{'PortalConfigFolder'},
        [
            {'string' => 'PortalConfigs'},
            {'hashref' => { 'Position' => 180 }}
        ]
    );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPortalConfigFolder {
    my $self = shift;
    my ($hParams) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object',{});

    $MockifyObject->addMockWithReturnValueAndParameterCheck(
        'existsChild',
        $hParams->{'IsDuplicatedPortalFolder'},
        [ {'string' => PORTAL_ALIAS} ]
    );

    return $MockifyObject->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createUniversalPortalSite {
    my $self = shift;
    my ($hParams) = @_;

    my $DateInThePast = SubtractDuration(GetCurrentDateTime(), 'months', 1);
    my $DeprecationDate = $hParams->{'IsPortalSiteDeprecated'} ? $DateInThePast : undef;
    my $hAttributes = {
        'Alias' => PORTAL_ALIAS,
        'DeprecationDate' => $DeprecationDate,
        'ID' => PORTAL_SITE_ID,
        'instanceOf' => $hParams->{'PortalSiteInstanceOf'},
        'LocaleID' => 'de_DE',
        'Portal' => PORTAL_LOCALEID,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::UniversalPortal::API::Object::UniversalPortalSite',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
